SystemOrganization addCategory: #'PhysicalEtoys-SerialPort'!
SystemOrganization addCategory: #'PhysicalEtoys-CompilerSupport'!
SystemOrganization addCategory: #'PhysicalEtoys-Installer'!
SystemOrganization addCategory: #'PhysicalEtoys-Projects'!
SystemOrganization addCategory: #'PhysicalEtoys-FileList'!
SystemOrganization addCategory: #'PhysicalEtoys-Gettext'!
SystemOrganization addCategory: #'PhysicalEtoys-Graphing'!
SystemOrganization addCategory: #'PhysicalEtoys-Spreadsheet'!
SystemOrganization addCategory: #'PhysicalEtoys-Utilities'!
SystemOrganization addCategory: #'PhysicalEtoys-CSV'!

NCAAConnectorMorph subclass: #PEWireMorph
	instanceVariableNames: 'isControllingConnections'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-Utilities'!

!PEWireMorph commentStamp: 'Richo 9/15/2009 12:22' prior: 0!
Abstract class. I represent the wire which connects different etoys. The difference with my superclass is that I control that the connections are valid by checking the classes of my sourceMorph and destinationMorph. Subclasses have to override #validConnectionClasses (see class side).!

!PEWireMorph class methodsFor: 'as yet unclassified' stamp: 'Richo 4/6/2011 10:47'!
defaultNameStemForInstances
^'Wire' translatedNoop! !

!PEWireMorph class methodsFor: 'instance creation' stamp: 'Richo 3/9/2010 10:28'!
fromMorph: aMorph toMorph: anotherMorph
	^( (super
		fromMorph: aMorph
		toMorph: anotherMorph
		) ) startControllingConnections! !

!PEWireMorph class methodsFor: 'constants' stamp: 'Richo 11/10/2009 12:25'!
validConnectionClasses
"Returns an array of valid classes to connect. Subclasses may override this method"
^{PEAttachmentPointMorph}! !

!PEWireMorph methodsFor: 'menu' stamp: 'Richo 3/9/2010 10:26'!
addCustomMenuItems: aMenu hand: aHand 
"This avoids adding connectors option to the menu"

	^ aMenu! !

!PEWireMorph methodsFor: 'connection control' stamp: 'Richo 3/9/2010 10:26'!
controlConnections

"Do not erase, it's very important!! It checks that the connections are valid and if they aren't it deletes itself"

(self hasValidClass: self sourceMorph) ifFalse: [self isInWorld ifTrue: [self delete]].
(self hasValidClass: self destinationMorph) ifFalse: [self isInWorld ifTrue: [self delete]].

Cursor normal show.! !

!PEWireMorph methodsFor: 'no se si sirve' stamp: 'Richo 10/7/2008 15:25'!
handlesMouseOver: evt 
	^ false! !

!PEWireMorph methodsFor: 'no se si sirve' stamp: 'Richo 10/7/2008 15:25'!
handlesMouseOverDragging: evt 
	^ false! !

!PEWireMorph methodsFor: 'as yet unclassified' stamp: 'Richo 9/8/2009 13:47'!
hasValidClass: aMorph 
	self class validConnectionClasses
		do: [:each | (aMorph isKindOf: each)
				ifTrue: [^ true]].
	^ false! !

!PEWireMorph methodsFor: 'initialization' stamp: 'Richo 3/9/2010 10:28'!
initialize
"isControllingConnections must be initialized first, otherwise it throws an error because somewhere #connectionsChanged gets called.
An alternative would be to use lazy initialization."
isControllingConnections := false.
super initialize.! !

!PEWireMorph methodsFor: 'testing' stamp: 'Richo 2/11/2009 13:44'!
isPlugged
	^ self sourceMorph notNil
		and: [self destinationMorph notNil]! !

!PEWireMorph methodsFor: 'connection control' stamp: 'Richo 9/1/2009 11:53'!
startControllingConnections
isControllingConnections := true.! !

!PEWireMorph methodsFor: 'stepping' stamp: 'Richo 11/10/2009 12:08'!
step
super step.

isControllingConnections ifTrue: [self controlConnections].! !

!PEWireMorph methodsFor: 'stepping' stamp: 'Richo 11/10/2009 12:08'!
stepTime
^50! !

!PEWireMorph methodsFor: 'connection control' stamp: 'Richo 9/1/2009 11:53'!
stopControllingConnections
isControllingConnections := false.! !

!PEWireMorph methodsFor: 'no se si sirve' stamp: 'Richo 10/7/2008 15:07'!
understandsBorderVocabulary
^false! !

!PEWireMorph methodsFor: 'no se si sirve' stamp: 'Richo 10/7/2008 18:21'!
wantsConnectionVocabulary
	^ false! !

!PEWireMorph methodsFor: 'no se si sirve' stamp: 'Richo 10/7/2008 15:04'!
wantsConnectorVocabulary
	^ false! !

!PEWireMorph methodsFor: 'no se si sirve' stamp: 'Richo 10/7/2008 18:21'!
wantsEmbeddingsVocabulary
	^ false! !

!PEWireMorph methodsFor: 'as yet unclassified' stamp: 'Richo 9/8/2009 13:46'!
wantsToAttachEnd: index toMorph: aMorph
^self hasValidClass: aMorph ! !

!Dictionary methodsFor: '*physicaletoys-csv' stamp: 'Richo 9/9/2011 15:15'!
asCSVString
	^ self asCSVStringWithDelimiter: $,! !

!Dictionary methodsFor: '*physicaletoys-csv' stamp: 'Richo 9/9/2011 15:18'!
asCSVStringWithDelimiter: aCharacter 
	| enclose |
	enclose := [:string | '"', (string copyReplaceAll: '"' with: '""'),'"'].
	^ String streamContents: [:stream || beforeFirst | 
			beforeFirst := true.
			self keysAndValuesDo: [:key :value |
				beforeFirst
					ifTrue: [beforeFirst := false]
					ifFalse: [stream cr].
				stream 
					nextPutAll: (enclose value: key asString);
					nextPut: aCharacter;
					nextPutAll: (enclose value: value asString)]]! !

ScriptStatusControl subclass: #PEScriptStatusControl
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-CompilerSupport'!

PEScriptStatusControl class
	instanceVariableNames: 'downloadButtonForm'!

PEScriptStatusControl class
	instanceVariableNames: 'downloadButtonForm'!

!PEScriptStatusControl class methodsFor: 'as yet unclassified' stamp: 'Richo 8/5/2011 17:06'!
downloadButtonForm
	"
	downloadButtonForm := nil.
	"
	^ downloadButtonForm ifNil: [downloadButtonForm := (Form
	extent: 28@28
	depth: 16
	fromArray: #( 0 0 0 0 0 787557104 787557104 787557104 787557104 0 0 0 0 0 0 0 0 10993 718285553 1677222904 1677222904 1677222904 1677222904 718285553 718274560 0 0 0 0 0 0 787571704 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677208304 0 0 0 0 0 718285553 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 718285553 0 0 0 10992 787571704 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677208304 787546112 0 0 10993 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 718274560 0 0 787571704 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677208304 0 10993 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 718274560 10992 1677222904 1677222904 1677222904 1677222904 1677209067 770387435 770387435 770401272 1677222904 1677222904 1677222904 1677222904 787546112 10993 1677222904 1677222904 1677222904 1677222904 1677209067 770387435 770387435 770401272 1677222904 1677222904 1677222904 1677222904 718274560 787571704 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677208304 718300152 1677222904 1677222904 1677222904 1677222904 1677209067 770387435 770387435 770401272 1677222904 1677222904 1677222904 1677222904 1677208305 787571704 1677222904 1677222904 1677222904 1677222904 1677209067 770387435 770387435 770401272 1677222904 1677222904 1677222904 1677222904 1677208304 718300152 1677222904 1677222904 1677222904 1677222904 1677209067 770387435 770387435 770401272 1677222904 1677222904 1677222904 1677222904 1677208305 787571704 1677222904 1677222904 1677222904 1677222904 1677209067 770387435 770387435 770401272 1677222904 1677222904 1677222904 1677222904 1677208304 718300152 1677222904 1677222904 1677222904 1677222904 1677209067 770387435 770387435 770401272 1677222904 1677222904 1677222904 1677222904 1677208305 787571704 1677222904 1677222904 1677222904 1677209067 770387435 770387435 770387435 770387435 770401272 1677222904 1677222904 1677222904 1677208304 718300152 1677222904 1677222904 1677222904 1677222904 770387435 770387435 770387435 770387435 1677222904 1677222904 1677222904 1677222904 1677208305 10992 1677222904 1677222904 1677222904 1677222904 1677209067 770387435 770387435 770401272 1677222904 1677222904 1677222904 1677222904 787546112 10993 1677222904 1677222904 1677222904 1677222904 1677222904 770387435 770387435 1677222904 1677222904 1677222904 1677222904 1677222904 718274560 10992 1677222904 1677222904 1677222904 1677222904 1677222904 1677209067 770401272 1677222904 1677222904 1677222904 1677222904 1677222904 787546112 0 718300152 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677208305 0 0 10992 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 787546112 0 0 10993 718300152 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677208305 718274560 0 0 0 787557104 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 787557104 0 0 0 0 0 718300152 1677222904 1677222904 1677222904 1677222904 1677222904 1677222904 1677208305 0 0 0 0 0 0 10992 787557104 1677222904 1677222904 1677222904 1677222904 787557104 787546112 0 0 0 0 0 0 0 0 718285553 718285553 718285553 718285553 0 0 0 0 0)
	offset: 0@0)]! !

!PEScriptStatusControl methodsFor: 'as yet unclassified' stamp: 'Richo 8/5/2011 16:51'!
assurePauseTickControlsShow
	"Add two little buttons that allow the user quickly to toggle between paused and ticking state"
	self scriptInstantiation player isUsingCompiledMode ifFalse: [^super assurePauseTickControlsShow].
	self beTransparent.
	(tickPauseWrapper isNil) ifTrue:[
		tickPauseWrapper _ SketchMorph withForm: self class downloadButtonForm.

		tickPauseWrapper on: #mouseUp send: #mouseUpTick:onItem: to: self.
		tickPauseWrapper on: #mouseDown send: #value to: [tickPauseWrapper form floodFill: Color green at: 5@5. tickPauseWrapper changed.].
		
		tickPauseWrapper setBalloonText:'Press to download this script.' translated.
		self addMorphFront: tickPauseWrapper.
	].
	tickPauseButtonsShowing _ true.! !

!PEScriptStatusControl methodsFor: 'as yet unclassified' stamp: 'Richo 8/5/2011 16:51'!
initializeFor: aScriptInstantiation
	"Answer a control that will serve to reflect and allow the user to change the status of the receiver"

	|  statusReadout |
	aScriptInstantiation player isUsingCompiledMode ifFalse: [^super initializeFor: aScriptInstantiation].
	self hResizing: #shrinkWrap.
	self vResizing: #shrinkWrap.
	self cellInset: 0@0.
	self layoutInset: 0.
	scriptInstantiation _ aScriptInstantiation.
	tickPauseButtonsShowing _ false.

	self addMorphBack: (statusReadout _ UpdatingSimpleButtonMorph new).
	statusReadout label: 'Richo capo' font: ScriptingSystem fontForEToyButtons.
	statusReadout setNameTo: 'trigger'.
	statusReadout height: statusReadout height - 4.
	statusReadout vResizing: #rigid.

	"statusReadout target: aScriptInstantiation; wordingSelector: #translatedStatus; actionSelector: #yourself.
	statusReadout setBalloonText: 'when this script should run' translated."
	statusReadout actWhen: #buttonDown; target: aScriptInstantiation; actionSelector: #presentTickingMenu.

	self assurePauseTickControlsShow.
	aScriptInstantiation updateStatusMorph: self! !

!PEScriptStatusControl methodsFor: 'as yet unclassified' stamp: 'Richo 3/30/2012 18:52'!
mouseUpTick: evt onItem: aMorph 
	| morph scriptName program player |
	
	[player := self scriptInstantiation player.
	player isUsingCompiledMode
		ifFalse: ["Something went wrong. 
			Probably the device was detached."
			player updateScriptEditors.
			^ self].
	morph := player costume renderedMorph.
	morph detectMainMorph isConnected
		ifFalse: [^ self].
	scriptName := self scriptInstantiation selector.
	program := morph translateScriptNamed: scriptName.
	Transcript show: program;
		 cr;
		 show: '----------------------------------------------------------------------';
		 cr. 
	Transcript
		show: (morph download: program);
		 cr;
		 show: '----------------------------------------------------------------------';
		 cr]
		ensure: [
			tickPauseWrapper isSketchMorph ifFalse: [^ self].
			tickPauseWrapper form floodFill: (Color r: 0.774 g: 1.0 b: 0.774) at: 5@5.
			tickPauseWrapper changed].! !

!NaturalLanguageTranslator class methodsFor: '*physicaletoys' stamp: 'Richo 9/9/2011 16:16'!
translate: aString toLocaleID: localeID inDomain: aDomainName
	^ PETranslator 
		translate: aString
		to: localeID
		ifError: [(self availableForLocaleID: localeID)
			translate: aString inDomain: aDomainName]
! !

FileList2 subclass: #PEFileList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-Projects'!

!PEFileList class methodsFor: 'as yet unclassified' stamp: 'Richo 3/7/2012 12:27'!
buildLoadButtons: window fileList: aFileList reallyLoad: aBoolean 
	| aRow  okButton cancelButton |
	okButton := self
				buildButtonText: 'OK' translated
				balloonText: nil
				receiver: aFileList
				selector: (aBoolean
						ifTrue: [#okHitForProjectLoader]
						ifFalse: [#okHit]).
	okButton width: 150.
	cancelButton := self
				buildButtonText: 'Cancel' translated
				balloonText: nil
				receiver: aFileList
				selector: #cancelHit.
	cancelButton width: 150.

	aRow := window addARow: {okButton. cancelButton}.
	aRow color: ScriptingSystem paneColor.
	aRow listCentering: #bottomRight.
	aRow layoutInset: 3 @ 3.
	aRow cellInset: 6 @ 3.
	^ aRow! !

!PEFileList class methodsFor: 'as yet unclassified' stamp: 'Richo 3/7/2012 12:41'!
buildSaveButtons: window fileList: aFileList
	| buttonData buttons aRow |
	buttonData := Preferences enableLocalSave
				ifTrue: [#(#('Save' #okHit 'Save in the place specified above')
							#('Save on local disk only' #saveLocalOnlyHit 'saves in the Squeaklets folder')
							#('Cancel' #cancelHit 'return without saving') ) translatedNoop]
				ifFalse: [#(#('Save' #okHit 'Save in the place specified above')
							#('Cancel' #cancelHit 'return without saving') ) translatedNoop].
	buttons := buttonData
				collect: [:each | self
						buildButtonText: each first translated
						balloonText: each third translated
						receiver: aFileList
						selector: each second].
	buttons := {Morph new color: Color transparent; width: 50}, buttons.
	aRow := window addARow: buttons.
	aRow color: ScriptingSystem paneColor.
	aRow listCentering: #bottomRight.
	aRow layoutInset: 3 @ 3.
	aRow cellInset: 6 @ 3.
	^ aRow! !

!PEFileList class methodsFor: 'as yet unclassified' stamp: 'Richo 3/7/2012 12:43'!
modalFolderSelectorForProject: aProject
"
FileList2 modalFolderSelectorForProject: Project current
"
	| window fileModel w |

	window _ self morphicViewProjectSaverFor: aProject.
	fileModel _ window valueOfProperty: #FileList.
	w _ self currentWorld.
	window position: w topLeft + (w extent - window extent // 2).
	w addMorphInLayer: window.
	w startSteppingSubmorphsOf: window.
	[window world notNil] whileTrue: [
		window outermostWorldMorph doOneCycle.
	].
	^fileModel getSelectedDirectory withoutListWrapper! !

!PEFileList class methodsFor: 'as yet unclassified' stamp: 'Richo 3/7/2012 12:45'!
morphicViewProjectSaverFor: aProject
"
(FileList2 morphicViewProjectSaverFor: Project current) openInWorld
"
	| window aFileList treePane pane2 option treeExtent row p |

	aFileList _ self new directory: ServerDirectory projectDefaultDirectory.
	aFileList dirSelectionBlock: self hideSqueakletDirectoryBlock.
	window := self buildMorphicWindow: aFileList title: 'Publish This Project' translated. 
	window setProperty: #morphicLayerNumber toValue: 11.

	option _ #limitedSuperSwikiDirectoryList.

	World height < 500 ifTrue: [
		treeExtent _ 350@150.
	] ifFalse: [
		treeExtent _ 350@300.
	].

	(treePane _ aFileList morphicDirectoryTreePaneFiltered: option) 
		extent: treeExtent; 
		retractable: false;
		borderWidth: 0.
treePane color: Color transparent.

	row := window addARow: {
			window inAColumn: {(ProjectViewMorph on: aProject) lock}.
			(
				window inAColumn: {
					(pane2 _ window inARow: {window inAColumn: {treePane}}) 
						useRoundedCornersInEtoys; layoutInset: 3
				}
			) layoutInset: 3
		}.
	row color: ScriptingSystem paneColor.
	pane2 borderColor: ScriptingSystem borderColor.
	pane2 borderWidth: 1.
	window addMorphBack: (self buildSaveButtons: window fileList: aFileList).
	window fullBounds.
	window beSticky.
	"This crazy stuff I really cannot figure out how to get the directory selected by default other than this."
	(treePane scroller submorphs detect: [:e |
		p := e complexContents withoutListWrapper pathName.
		(p beginsWith: 'sugar://') or: [p = SecurityManager default untrustedUserDirectory]] ifNone: [nil])
			ifNotNilDo: [:item | WorldState addDeferredUIMessage: [aFileList setSelectedDirectoryTo: item complexContents]].
	aFileList postOpen.
	^ window ! !

!PEFileList class methodsFor: 'as yet unclassified' stamp: 'Richo 3/7/2012 12:23'!
projectOnlySelectionMethod: incomingEntries

	| versionsAccepted basicInfoTuple basicName basicVersion |

	"this shows only the latest version of each project"
	versionsAccepted _ Dictionary new.
	incomingEntries do: [ :entry |
		entry isDirectory ifFalse: [
			(#('*.pe' '*.pr' '*.pr.gz' '*.project') anySatisfy: [ :each | each match: entry name]) ifTrue: [
				basicInfoTuple _ Project parseProjectFileName: entry name.
				basicName _ basicInfoTuple first.
				basicVersion _ basicInfoTuple second.
				((versionsAccepted includesKey: basicName) and: 
						[(versionsAccepted at: basicName) first > basicVersion]) ifFalse: [
					versionsAccepted at: basicName put: {basicVersion. entry}
				].
			]
		]
	].
	^versionsAccepted asArray collect: [ :each | each second]! !

!PEFileList methodsFor: 'as yet unclassified' stamp: 'Richo 3/7/2012 12:24'!
limitedSuperSwikiDirectoryList
	| dirList |
	dirList _ (FileDirectory on: '') directoryNames collect: [ :each |
		FileDirectoryWrapper with: (FileDirectory on: each) name: each model: self].
	dirList isEmpty ifTrue:[
		dirList _ Array with: (FileDirectoryWrapper 
			with: FileDirectory default 
			name: FileDirectory default localName 
			model: self)].
	dirList do: [:e | e balloonText: e withoutListWrapper pathName].
	^dirList! !

!PEFileList methodsFor: 'as yet unclassified' stamp: 'Richo 3/7/2012 12:21'!
openProjectFromFile
	"Reconstitute a Morph from the selected file, presumed to be represent
	a Morph saved via the SmartRefStream mechanism, and open it in an
	appropriate Morphic world."
	(fileName endsWith: '.pe')
		ifTrue: [PEProject
			openProjectNamed: fileName
			on: currentDirectorySelected withoutListWrapper]
		ifFalse: [ProjectViewMorph 
			openFromDirectory: directory 
			andFileName: fileName]! !

!Collection methodsFor: '*physicaletoys' stamp: 'Richo 1/10/2012 14:14'!
flattened
	^ Array
		streamContents: [:stream | self
				do: [:each | (each isCollection and: [each isString not])
								ifTrue: [stream nextPutAll: each flattened]
								ifFalse: [stream nextPut: each]]]! !

!ScriptInstantiation methodsFor: '*physicaletoys-compilersupport' stamp: 'Richo 8/5/2011 16:51'!
statusControlMorph
	"Answer a control that will serve to reflect (and allow the user to change) the status of the receiver"
	self player isUsingCompiledMode 
		ifTrue: [^ PEScriptStatusControl new initializeFor: self]
		ifFalse: [^ ScriptStatusControl new initializeFor: self]
! !

!ScriptInstantiation methodsFor: '*physicaletoys-compilersupport' stamp: 'Richo 2/29/2012 13:24'!
updateStatusMorph: statusControlMorph
	"the status control may need to reflect an externally-induced change in status"

	| colorSelector statusReadoutButton |
	statusControlMorph ifNil: [^ self].
	self pausedOrTicking
		ifTrue:
			[statusControlMorph assurePauseTickControlsShow]
		ifFalse:
			[statusControlMorph maybeRemovePauseTickControls].
	self player isUsingCompiledMode
		ifTrue: [""
			(statusControlMorph isKindOf: PEScriptStatusControl)
				ifFalse: ["The player is using compiled mode but this script editor
						was not updated somehow"
					self player updateScriptEditors.
					^ self].
			statusReadoutButton _ statusControlMorph submorphs last.
			colorSelector _ #green.
			statusReadoutButton color: (Color perform: colorSelector) muchLighter.
			statusReadoutButton label: ('{1} ticks / sec' format: {self tickingRate}) font: ScriptingSystem fontForEToyButtons.
		^self].
	statusReadoutButton _ statusControlMorph submorphs last.
	colorSelector _ ScriptingSystem statusColorSymbolFor: self status.
	statusReadoutButton color: (Color perform: colorSelector) muchLighter.
	statusReadoutButton label: self translatedStatus asString font: ScriptingSystem fontForEToyButtons! !

CircleMorph subclass: #PEAttachmentPointMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-Utilities'!

!PEAttachmentPointMorph commentStamp: 'Richo 3/9/2010 10:20' prior: 0!
I exist only for esthetic purposes. I make sure that wires connecting devices are attached to the correct points (not the center) of the morphs.

I think Connectors provides a way of doing this without a submorph but I couldn't find it.!

!PEAttachmentPointMorph methodsFor: 'as yet unclassified' stamp: 'Richo 9/1/2009 16:07'!
disconnectedFromBy: aConnector

owner disconnectedFromBy: aConnector.! !

RBProgramNodeVisitor subclass: #PlayerTranslator
	instanceVariableNames: 'getterSelectors scriptSelectors setterSelectors currentMorph variables methods mainMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-CompilerSupport'!

!PlayerTranslator commentStamp: 'Richo 3/30/2012 17:19' prior: 0!
This class is responsible for merging and transforming a bunch of Etoys scripts (which may be compiled in a set of different Player subclasses) into one class. The details of this merging should be defined in subclasses.

This process usually requires:
* compiling all methods and variables into one single class (adding a prefix representing the receiver to avoid name clashes).
* renaming the message selectors according to rules defined by subclasses.
* changing all receivers to self.

Apart from that, it also takes care of representing etoys multithreading using an eternal loop that runs each script whenever is its turn. To control which script is ticking, it defines boolean inst vars that can be set to true or false to start and pause/stop each script individually.

See #translateMainScript:of: as a starting point on how to use this class. Also, see subclasses for actual implementation.
!

!PlayerTranslator class methodsFor: 'instance creation' stamp: 'Richo 3/30/2012 18:33'!
on: aMorph
	^ self basicNew 
		initializeWith: aMorph;
		yourself! !

!PlayerTranslator methodsFor: 'visiting' stamp: 'Richo 3/30/2012 17:33'!
acceptMessageNode: aMessageNode 
	| selector |
	selector := aMessageNode selector.
	"Check for invalid selector or invalid receiver"
	self assert: [self isValidSelector: selector];
		assert: [self isValidReceiver: aMessageNode receiver].
	"Is it a rule?"
	(self rules includesKey: selector)
		ifTrue: [^(self rules at: selector)
				value: aMessageNode].
	"Is it an etoys message?"
	(self etoysMessages includesKey: selector)
		ifTrue: [(self etoysMessages at: selector)
				value: aMessageNode].
	"Is it a getter?"
	(self allGetterSelectors includes: selector)
		ifTrue: [| newNode | 
			self
				withCurrentMorph: (self morphFor: aMessageNode receiver name)
				do: [newNode := self variable: (self slotForGetter: selector)].
			aMessageNode
				become: (newNode parent: aMessageNode parent).
			^ self visitNode: aMessageNode].
	"Is it a setter?"
	(self allSetterSelectors includes: selector)
		ifTrue: [| newNode | 
			self
				withCurrentMorph: (self morphFor: aMessageNode receiver name)
				do: [newNode := self assignment: (self variable: (self slotForSetter: selector))
										to: aMessageNode arguments first].
			aMessageNode
				become: (newNode parent: aMessageNode parent).
			^ self visitNode: aMessageNode].
	"Is it a script call?"
	(self allScriptSelectors includes: selector)
		ifTrue: [self
				withCurrentMorph: (self morphFor: aMessageNode receiver name)
				do: ["Check for recursive call"
					self assert: [self isNotRecursiveCall: aMessageNode].
					aMessageNode
						selector: (self asLegalName: aMessageNode selector);
						
						receiver: (self variable: 'self')]].
	"We still need to visit the receiver and the arguments"
	^ super acceptMessageNode: aMessageNode! !

!PlayerTranslator methodsFor: 'visiting' stamp: 'Richo 3/30/2012 17:33'!
acceptMethodNode: aMethodNode 
	"It goes over the method's arguments adding the #argTypes: pragma if necessary and modifies the selector to have a legal name"
	aMethodNode allArgumentVariables size = 1 
		ifTrue: [| varType | 
			varType := self asValidType: (self currentPlayerClass scripts at: aMethodNode selector) argumentVariables first variableType.
			aMethodNode pragmas: (RBParser parseMethod: 'm <argTypes: #(' , varType , ')>') pragmas].
	aMethodNode
		selector: (self asLegalName: aMethodNode selector).
	^ super acceptMethodNode: aMethodNode! !

!PlayerTranslator methodsFor: 'visiting' stamp: 'Richo 3/30/2012 17:33'!
acceptSequenceNode: aSequenceNode
	"If one of the statements has an invalid tile it will ignore it entirely. Only assertion failures will be ignored, other errors will be signaled, though"
	aSequenceNode statements do: [:each | [self visitNode: each] on: AssertionFailure do: [self becomeNoOp: each]]
! !

!PlayerTranslator methodsFor: 'accessing' stamp: 'Richo 3/30/2012 17:33'!
allGetterSelectors
	^ getterSelectors inject: #() into: [:subTotal :next | subTotal, next]
! !

!PlayerTranslator methodsFor: 'accessing' stamp: 'Richo 3/30/2012 17:34'!
allMorphs
	"Returns the collection of morphs whose players I need to merge.
	It could contain nils"
	^ self subclassResponsibility! !

!PlayerTranslator methodsFor: 'iterating' stamp: 'Richo 3/30/2012 17:34'!
allMorphsDo: aBlock
	"Iterates over all morphs composing the robot taking care of posible nil occurrences"
	self allMorphs do: [:each |
		each notNil ifTrue: [aBlock value: each]]! !

!PlayerTranslator methodsFor: 'accessing' stamp: 'Richo 3/30/2012 18:37'!
allPrimitiveRulesSelectors
	"Returns all messages in the CProgram hierarchy that the program
	is able to send"
	| current result |
	result := Set new.
	"Collect all selectors in the common superclass protocol..."
	current := self commonSuperclass.
	[result addAll: current selectors]
		doWhileTrue: [(current := current superclass) ~= CObject superclass].
	"... and in all the primitive types' protocol as well"
	CPrimitiveType withAllSubclassesDo: [:class |
		result addAll: class selectors].
	^ result! !

!PlayerTranslator methodsFor: 'accessing' stamp: 'Richo 3/30/2012 17:35'!
allScriptSelectors
	^ scriptSelectors inject: #() into: [:subTotal :next | subTotal, next asArray]
! !

!PlayerTranslator methodsFor: 'accessing' stamp: 'Richo 3/30/2012 17:37'!
allSetterSelectors
	^ setterSelectors inject: #() into: [:subTotal :next | subTotal, next asArray]
! !

!PlayerTranslator methodsFor: 'converting' stamp: 'Richo 4/9/2012 17:09'!
asLegalName: aString
	"All variables, method names and task names must begin with a prefix representing the receiver."
	^ currentMorph assuredPlayer uniqueNameForReference asLegalSelector, 
		aString capitalized! !

!PlayerTranslator methodsFor: 'converting' stamp: 'Richo 3/30/2012 17:39'!
asLegalScriptFlag: aSymbol 
	"Returns the name of the flag variable used to stop this script"
	^ self asLegalName: (self asScriptFlag: aSymbol)! !

!PlayerTranslator methodsFor: 'converting' stamp: 'Richo 3/30/2012 17:38'!
asLegalScriptLastTime: aByteSymbol 
	^ self asLegalName: (self asScriptLastTime: aByteSymbol)
! !

!PlayerTranslator methodsFor: 'converting' stamp: 'Richo 3/30/2012 17:39'!
asScriptFlag: aSymbol
	"Returns the name of the flag variable used to stop this script"
	^ (self asScriptName: aSymbol), 'Stepping'! !

!PlayerTranslator methodsFor: 'converting' stamp: 'Richo 3/30/2012 17:39'!
asScriptLastTime: aSymbol
	^ aSymbol , 'LastTime'! !

!PlayerTranslator methodsFor: 'converting' stamp: 'Richo 3/30/2012 17:39'!
asScriptName: aSymbol
	"Returns the name of this script"
	^ aSymbol, 'Script'! !

!PlayerTranslator methodsFor: 'converting' stamp: 'Richo 3/30/2012 17:39'!
asValidType: aSymbol
	^ aSymbol caseOf: {
		[#Number] -> [#CLong].
		[#Boolean] -> [#CBoolean].
		[#String] -> [#CString]
		} otherwise: [aSymbol]! !

!PlayerTranslator methodsFor: 'creating parse nodes' stamp: 'Richo 3/30/2012 17:39'!
assignment: aRBVariableNode to: aRBLiteralValueNode 
	^ RBAssignmentNode
		variable: aRBVariableNode
		value: aRBLiteralValueNode! !

!PlayerTranslator methodsFor: 'converting' stamp: 'Richo 3/30/2012 17:40'!
becomeNoOp: aParseNode
	"Transforms the argument in an empty statement:
	
	self c: ''.
	"
	| noOp |
	noOp := RBMessageNode receiver: (self variable: 'self') selector: #c: arguments: {self literal: ''}.
	aParseNode become: (noOp parent: aParseNode parent)! !

!PlayerTranslator methodsFor: 'accessing' stamp: 'Richo 3/30/2012 17:55'!
classCategory
	"Should return the category in which generate the class"
	self subclassResponsibility! !

!PlayerTranslator methodsFor: 'accessing' stamp: 'Richo 3/30/2012 17:58'!
classIncludes
	"Should return an array with the name of the libraries used by
	this program"
	^ self subclassResponsibility ! !

!PlayerTranslator methodsFor: 'initializing' stamp: 'Richo 3/30/2012 17:41'!
collectGetterSelectors
	"Collects all selectors corresponding to a slot getter in a dictionary whose keys are the morphs owning the slot"
	getterSelectors := Dictionary new.
	self allMorphsDo: [:morph | | selectors |
		selectors := getterSelectors at: morph ifAbsentPut: [OrderedCollection new].
		selectors addAll: (morph assuredPlayer slotInfo keys collect: [:selector | Utilities getterSelectorFor: selector])].! !

!PlayerTranslator methodsFor: 'initializing' stamp: 'Richo 3/30/2012 17:44'!
collectMorphs
	"Finds all posible morphs whose players I should translate"
	self subclassResponsibility! !

!PlayerTranslator methodsFor: 'initializing' stamp: 'Richo 3/30/2012 17:42'!
collectRelevantInformation
	self collectMorphs; 
		 collectSlots;
		 collectGetterSelectors;
		 collectSetterSelectors;
		 collectScriptSelectors! !

!PlayerTranslator methodsFor: 'initializing' stamp: 'Richo 3/30/2012 17:45'!
collectScriptSelectors
	"Collects all selectors corresponding to a script in a dictionary whose keys are the morphs owning the methods"
	scriptSelectors := Dictionary new.
	self
		allMorphsDo: [:morph | 
			| selectors | 
			selectors := scriptSelectors
						at: morph
						ifAbsentPut: [Set new].
			selectors
				addAll: (morph assuredPlayer class selectors
						reject: [:selector | ((getterSelectors at: morph)
									includes: selector)
								or: [(setterSelectors at: morph)
										includes: selector]])] ! !

!PlayerTranslator methodsFor: 'initializing' stamp: 'Richo 3/30/2012 17:45'!
collectSetterSelectors
	"Collects all selectors corresponding to a slot setter in a dictionary whose keys are the morphs owning the slot"
	setterSelectors := Dictionary new.
	self allMorphsDo: [:morph | | selectors |
		selectors := setterSelectors at: morph ifAbsentPut: [OrderedCollection new].
		selectors addAll: (morph assuredPlayer slotInfo keys collect: [:selector | Utilities setterSelectorFor: selector])].
! !

!PlayerTranslator methodsFor: 'initializing' stamp: 'Richo 3/30/2012 17:46'!
collectSlots
	"Adds all the slots and they respective types to the variables dictionary."
	variables := Dictionary new.
	self allMorphsDo: [:morph | | slots |
		slots := variables at: morph ifAbsentPut: [Dictionary new].
		morph assuredPlayer slotInfo 
			keysAndValuesDo: [:slotName :slotInfo | 
				slots at: slotName put: (slotInfo type -> (morph assuredPlayer instVarNamed: slotName))]]
! !

!PlayerTranslator methodsFor: 'accessing' stamp: 'Richo 3/30/2012 17:55'!
commonSuperclass
	"Should return the superclass for the generated class"
	self subclassResponsibility! !

!PlayerTranslator methodsFor: 'translating' stamp: 'Richo 4/9/2012 17:35'!
compileLoopMethod
	methods add: (String streamContents: [:stream |
		stream nextPutAll: 'loop
	| now |
	<tempTypes: #(CUnsignedLong)>
	now := self currentTick.
	'.
	self scriptSelectorsDo: [:selector || code |
		"Only control execution of scripts without arguments"
		(selector endsWith: ':')
			ifFalse: [
				code := '({1} and: [now - {2} > {3}])
								ifTrue: [{4} timesRepeat: [self {5}].
							{2} := now].' 
						format: {
							(self asLegalScriptFlag: selector).
							(self asLegalScriptLastTime: selector).
							(self delayOfScript: selector).
							(self frequencyOfScript: selector).
							(self asLegalName: selector)}.
				stream nextPutAll: code].
		]
	])! !

!PlayerTranslator methodsFor: 'translating' stamp: 'Richo 3/30/2012 17:53'!
compileSetUpMethod
	"Generates the #setup method, which gets executed once at the 
	beginning of the program and its 	responsible for initializing all 
	the inst vars and configuring devices and stuff.
	The generated string should be added to the 'methods' inst var."
	self subclassResponsibility
! !

!PlayerTranslator methodsFor: 'compiling' stamp: 'Richo 3/30/2012 17:53'!
compileTempProgramSubclass
	| instVarNames instVarTypes |
	instVarNames := ''.
	instVarTypes := #().
	"First collect all variables"
	self
		variablesDo: [:varName :varTypeAndValue | 
			instVarNames := instVarNames, (self asLegalName: varName) , ' '.
			instVarTypes := instVarTypes, {self asValidType: varTypeAndValue key}].
	"Then create the class"
	self createTempProgramSubclassInstVarNames: instVarNames
		 instVarTypes: instVarTypes.
	"Finally compile all methods"
	methods
		do: [:each | self programClass compileSilently: each classified: #autogenerated]
! !

!PlayerTranslator methodsFor: 'translating' stamp: 'Richo 4/10/2012 12:05'!
createControlVariablesFor: scriptSelector started: startedBoolean
	"Creates the variables to control the execution of the
	given script. 
	The started variable determines if the script should begin
	with its flag set to true."
	(variables at: currentMorph)
		at: (self asScriptFlag: scriptSelector)
		put: #Boolean -> startedBoolean.
	(variables at: currentMorph)
		at: (self asScriptLastTime: scriptSelector)
		put: #CUnsignedLong -> 0
! !

!PlayerTranslator methodsFor: 'compiling' stamp: 'Richo 3/30/2012 17:57'!
createTempProgramSubclassInstVarNames: aString instVarTypes: anArray 
	SystemChangeNotifier uniqueInstance doSilently: [
		self commonSuperclass subclass: self programName
			instanceVariableNames: aString
			instanceVariableTypes: anArray
			classVariableNames: ''
			poolDictionaries: ''
			category: self classCategory
			includes: self classIncludes]
! !

!PlayerTranslator methodsFor: 'accessing' stamp: 'Richo 3/30/2012 17:58'!
currentPlayer
	^ currentMorph assuredPlayer! !

!PlayerTranslator methodsFor: 'accessing' stamp: 'Richo 3/30/2012 17:58'!
currentPlayerClass
	^ self currentPlayer class! !

!PlayerTranslator methodsFor: 'accessing' stamp: 'Richo 3/30/2012 17:59'!
delayOfScript: scriptName
 	"To be in accordance with the script tick rate, we put a delay at the end of each task. How long the delay would be?"
	^ (1000 / (self currentPlayer scriptInstantiationForSelector: scriptName) tickingRate) rounded asString
! !

!PlayerTranslator methodsFor: 'accessing' stamp: 'Richo 3/30/2012 18:00'!
etoysMessages
	^ {
		"#grouped actually doesn't do anything so we just take it out"
		#grouped -> [:node | node become: (node receiver parent: node parent)].
	
		"I don't know why but recent versions of Etoys use this weird 
		messages instead of the comparison operators:"
		#eToysLT: -> [:node | node selector: #<].
		#eToysLE: -> [:node | node selector: #<=].
		#eToysEQ: -> [:node | node selector: #=].
		#eToysNE: -> [:node | node selector: #~=].
		#eToysGT: -> [:node | node selector: #>].
		#eToysGE: -> [:node | node selector: #>=].
	} as: Dictionary! !

!PlayerTranslator methodsFor: 'accessing' stamp: 'Richo 3/30/2012 18:00'!
frequencyOfScript: scriptName
	"To be in accordance with the script frequency property we enclose the script call in a #timesRepeat: message. How many times should we repeat the script?"
	^ (self currentPlayer scriptInstantiationForSelector: scriptName) frequency rounded asString
! !

!PlayerTranslator methodsFor: 'accessing' stamp: 'Richo 3/30/2012 18:00'!
hashString
	^ self hash asString
! !

!PlayerTranslator methodsFor: 'initializing' stamp: 'Richo 3/30/2012 18:01'!
initializeWith: anArduinoMorph
	"By default we set the main morph as the currentMorph"
	mainMorph := currentMorph := anArduinoMorph.
	self initialize! !

!PlayerTranslator methodsFor: 'visiting' stamp: 'Richo 3/30/2012 18:03'!
isNotRecursiveCall: aMessageNode
	^ (self asLegalName: aMessageNode selector) ~= aMessageNode methodNode selector! !

!PlayerTranslator methodsFor: 'testing' stamp: 'Richo 3/30/2012 18:05'!
isValidReceiver: aParseNode 
	"If the parse node is a variable it *must* be one of the valid morphs
	or an argument variable. I'm not sure about this, actually so we might 
	have to test it more"
	| res |
	aParseNode isVariable
		ifTrue: [res := (aParseNode methodNode allArgumentVariables
							anySatisfy: [:each | each = aParseNode name])
						or: [(self morphFor: aParseNode name) notNil].
			self flag: #Richo.
			^ res].
	^ true! !

!PlayerTranslator methodsFor: 'testing' stamp: 'Richo 3/30/2012 18:34'!
isValidSelector: selector 
	"Checks if the selector is one of the following:
	* A primitive.
	* A player translator rule
	* An etoys message.
	* A slot accessor.
	* A script selector."
	^ (self allPrimitiveRulesSelectors asSet, 
		self rules keys, 
		self etoysMessages keys,
		self allGetterSelectors,
		self allSetterSelectors, 
		self allScriptSelectors)
			includes: selector
! !

!PlayerTranslator methodsFor: 'creating parse nodes' stamp: 'Richo 3/30/2012 18:06'!
literal: aLiteralObject
	^ RBLiteralValueNode value: aLiteralObject! !

!PlayerTranslator methodsFor: 'accessing' stamp: 'Richo 3/30/2012 18:07'!
morphFor: receiverName
	"Returns the morph represented by the argument or nil if it's not a valid morph"
	^ receiverName = 'self'
		ifTrue: [currentMorph]
		ifFalse: [self allMorphs
				detect: [:each | each notNil 
					and: [each assuredPlayer uniqueNameForReference = receiverName]]
				ifNone: [nil]]
! !

!PlayerTranslator methodsFor: 'accessing' stamp: 'Richo 3/30/2012 18:08'!
programClass
	"Returns the temporary class we are using to compile the code"
	^ Smalltalk at: self programName ifAbsent: []! !

!PlayerTranslator methodsFor: 'accessing' stamp: 'Richo 3/30/2012 18:08'!
programName
	"Returns the name of the class we're using to compile the code"
	^ self subclassResponsibility ! !

!PlayerTranslator methodsFor: 'compiling' stamp: 'Richo 3/30/2012 18:09'!
removeTempProgramSubclass
	"Before translating we silently remove the temporary class, so that no garbage
	is left from a previous program"
	SystemChangeNotifier uniqueInstance doSilently: [
		self programClass ifNotNilDo: [:class | class removeFromSystem]]
! !

!PlayerTranslator methodsFor: 'accessing-rules' stamp: 'Richo 4/10/2012 12:28'!
rules
	"Should return a dictionary whose keys are the message selectors and whose 
	values are blocks accepting a node as a parameter. These blocks are the rules
	that modify the parse tree of the player scripts.
	Most of the time, all that is needed is to rename the selector, change the receiver 
	to 'self' and set the correct list of arguments. But the rules can be a little more 
	complicated sometimes"
	^ self scriptingRules as: Dictionary! !

!PlayerTranslator methodsFor: 'iterating' stamp: 'Richo 3/30/2012 18:15'!
scriptSelectorsDo: aBlock 
	"Iterates over the scriptSelectors dictionary taking care of setting the current morph"
	scriptSelectors
		keysAndValuesDo: [:morph :selectors | self
				withCurrentMorph: morph
				do: [selectors do: aBlock]] ! !

!PlayerTranslator methodsFor: 'accessing-rules' stamp: 'Richo 4/10/2012 12:23'!
scriptingRules
"This is the only set of rules shared by all subclasses because they all use the same execution control mechanism"

"For the tiles that receive a script name as a parameter, it first checks that the script actually belongs to the current morph, and if it doesn't then it just turns the node into a noop"
^ {
#startScript: -> [:node |
	self withCurrentMorph: (self morphFor: node receiver name) do: [| scriptName newNode |
		scriptName := node arguments first value.
		self assert: ((scriptSelectors at: currentMorph) includes: scriptName).
		newNode := self assignment: (self variable: (self asLegalScriptFlag: scriptName))
							to: (self literal: true).
		node become: (newNode parent: node parent)]].

#stopScript: -> [:node |
	self withCurrentMorph: (self morphFor: node receiver name) do: [| scriptName newNode |
		scriptName := node arguments first value.
		self assert: ((scriptSelectors at: currentMorph) includes: scriptName).
		newNode := self assignment: (self variable: (self asLegalScriptFlag: scriptName))
							to: (self literal: false).
		node become: (newNode parent: node parent)]].

#pauseScript: -> [:node | "Do the same as #stopScript:"
	(self rules at: #stopScript:) value: node].

#doScript: -> [:node | | args | args := node arguments. "This is the same as calling the script directly"
	node arguments: #(); selector: args first value.
	self visitNode: node "We need to visit the node again to fix the selector and the receiver"]
}! !

!PlayerTranslator methodsFor: 'accessing' stamp: 'Richo 3/30/2012 18:16'!
slotForGetter: getterSelector
	"Returns the slot accessed by these selector"
	| slotName |
	slotName := currentMorph assuredPlayer slotInfo keys
				detect: [:slot | (Utilities getterSelectorFor: slot)
						= getterSelector]
				ifNone: [nil].
	self assert: slotName notNil.
	^ self asLegalName: slotName! !

!PlayerTranslator methodsFor: 'accessing' stamp: 'Richo 3/30/2012 18:16'!
slotForSetter: setterSelector
	"Returns the slot accessed by these selector"
	| slotName |
	slotName := currentMorph assuredPlayer slotInfo keys
				detect: [:slot | (Utilities setterSelectorFor: slot)
						= setterSelector]
				ifNone: [nil].
	self assert: slotName notNil.
	^ self asLegalName: slotName! !

!PlayerTranslator methodsFor: 'translating' stamp: 'Richo 4/10/2012 12:08'!
translateMainScript: scriptName of: scriptOwner
	"Entry point for the PlayerTranslator. It creates a temporary subclass
	to hold the code which will be translated to c.

	Steps:
	* Collect relevant information (morphs composing the robot, slots, accessors and scripts).
	* Compile all scripts.
	* Compile #loop method.
	* Compile #setup method.
	* Remove temporary class (to discard garbage).	
	* Compile temporary class.
	* Return generated class"
	self collectRelevantInformation.
	self
		assert: [self allMorphs includes: scriptOwner]
		description: 'The morph is not connected to the main morph';
		assert: [scriptOwner assuredPlayer class scripts includesKey: scriptName]
		description: 'The script doesn''t belong to the morph'.
	methods := Set new. 
	self
		scriptSelectorsDo: [:selector | 
			| node | 
			self visitNode: (node := self currentPlayerClass parseTreeFor: selector).
			"Only create control variables for ticking scripts"
			(selector endsWith: ':')
				ifFalse: [self 
							createControlVariablesFor: selector
							started: (selector = scriptName
								and: [currentMorph = scriptOwner])].
			methods add: node formattedCode].
	self
		withCurrentMorph: scriptOwner
		do: [self compileLoopMethod; compileSetUpMethod].
	self removeTempProgramSubclass;
		compileTempProgramSubclass.
	^ self programClass! !

!PlayerTranslator methodsFor: 'creating parse nodes' stamp: 'Richo 4/9/2012 17:10'!
variable: variableName
	^ RBVariableNode named: variableName! !

!PlayerTranslator methodsFor: 'iterating' stamp: 'Richo 3/30/2012 18:16'!
variablesDo: aBlock 
	"Iterates over the variables dictionary taking care of setting the current morph"
	variables
		keysAndValuesDo: [:morph :vars | self
				withCurrentMorph: morph
				do: [vars keysAndValuesDo: aBlock]] ! !

!PlayerTranslator methodsFor: 'translating' stamp: 'Richo 3/30/2012 18:17'!
withCurrentMorph: aMorph do: aBlock
	"It executes the block using the morph argument as our current morph.
	It also remembers the old morph, so it's safe to nest this expression.
	The current morph is used whenever we need a legal name for a 
	variable/method name. It defines which morph is currently represented
	by the 'self' variable"
	| old |
	old := currentMorph.
	currentMorph := aMorph.
	aBlock ensure: [currentMorph := old]! !

!ScrollPane methodsFor: '*PhysicalEtoys' stamp: 'Richo 4/12/2011 12:05'!
defersHaloToInteriorMorph: aMorph
^true! !

!ScrollPane methodsFor: '*PhysicalEtoys' stamp: 'Richo 4/12/2011 15:14'!
handlesKeyboard: evt
	"This permits the use of the scroll button in the mouse"
	^true! !

Object subclass: #CSVParser
	instanceVariableNames: 'stream delimiter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-CSV'!

!CSVParser class methodsFor: 'instance creation' stamp: 'Richo 10/24/2011 01:07'!
on: stream delimiter: aCharacter
	^ self basicNew initializeWith: stream delimiter: aCharacter! !

!CSVParser class methodsFor: 'parsing' stamp: 'Richo 10/24/2011 01:07'!
parse: aStream
	^ self parse: aStream delimiter: $,! !

!CSVParser class methodsFor: 'parsing' stamp: 'Richo 10/24/2011 01:07'!
parse: aStream delimiter: aCharacter
	^ (self on: aStream delimiter: aCharacter) parse! !

!CSVParser class methodsFor: 'parsing' stamp: 'Richo 10/22/2011 19:07'!
parseString: aString
	^ self parse: aString readStream ! !

!CSVParser class methodsFor: 'parsing' stamp: 'Richo 10/24/2011 01:08'!
parseString: aString delimiter: aCharacter
	^ self parse: aString readStream delimiter: aCharacter! !

!CSVParser methodsFor: 'initialization' stamp: 'Richo 10/24/2011 01:08'!
initializeWith: aStream delimiter: aCharacter
	stream := aStream.
	delimiter := aCharacter.
	self initialize! !

!CSVParser methodsFor: 'parsing' stamp: 'Richo 10/24/2011 00:48'!
nextCell
	^ stream peek = $"
		ifTrue: [self nextEnclosedCell]
		ifFalse: [self nextNormalCell]! !

!CSVParser methodsFor: 'parsing' stamp: 'Richo 10/24/2011 10:13'!
nextEnclosedCell
	^ String streamContents: [:cell |
		stream skip: 1.
		[stream atEnd]
			whileFalse: [| next |
				next := stream next.
				next = $" 
					ifTrue: [
						stream peek ~= $"
							ifTrue: [^ cell contents]
							ifFalse: [stream skip: 1]].
				cell nextPut: next]]! !

!CSVParser methodsFor: 'parsing' stamp: 'Richo 10/24/2011 09:53'!
nextNormalCell
	^ String streamContents: [:cell |
		[stream atEnd]
			whileFalse: [| next |
				next := stream next.
				(next = delimiter
				or: [Character cr = next 
				or: [Character lf = next]])
					ifTrue: [stream skip: -1.
						^ cell contents].
				cell nextPut: next]]! !

!CSVParser methodsFor: 'parsing' stamp: 'Richo 10/24/2011 01:09'!
nextRow
	^ Array streamContents: [:cells |
		[stream atEnd or: [String crlf includes: stream peek]]
			whileFalse: [cells nextPut: self nextCell.
				stream peek = delimiter ifTrue: [stream skip: 1]]]! !

!CSVParser methodsFor: 'parsing' stamp: 'Richo 10/24/2011 01:22'!
parse
	^ Array streamContents: [:rows |
		[stream atEnd]
			whileFalse: [rows nextPut: self nextRow.
				[String crlf includes: stream peek] whileTrue: [stream skip: 1]]]! !

Object subclass: #PEInstaller
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-Installer'!

PEInstaller class
	instanceVariableNames: 'installedModules'!

PEInstaller class
	instanceVariableNames: 'installedModules'!

!PEInstaller class methodsFor: 'user interface' stamp: 'Richo 3/14/2012 18:50'!
inform: aString
	(PopUpMenu labels: 'OK' translated)
		startUpWithCaption: aString
		at: World center
		allowKeyboard: Preferences menuKeyboardControl
		centered: true! !

!PEInstaller class methodsFor: 'class initialization' stamp: 'Richo 3/7/2012 12:54'!
initialize
	"
	self initialize.
	"
	Smalltalk
		addToStartUpList: self! !

!PEInstaller class methodsFor: 'registering' stamp: 'Richo 3/14/2012 19:25'!
installedModules
	"
	installedModules := nil
	"
	^ installedModules ifNil: [installedModules := OrderedCollection new]! !

!PEInstaller class methodsFor: 'registering' stamp: 'Richo 3/13/2012 16:06'!
isAlreadyInstalled: fileName
	^ self installedModules includes: fileName! !

!PEInstaller class methodsFor: 'registering' stamp: 'Richo 3/13/2012 16:04'!
register: fileName
	self installedModules add: fileName! !

!PEInstaller class methodsFor: 'system startup' stamp: 'Richo 3/14/2012 18:46'!
searchAndInstall
	"Search the current directory for physical etoys modules and install them"
	| modules |
	modules := self searchForModules.
	modules isEmpty ifTrue: [^ self].
	[modules do: [:module |
		module install.
		self inform: ('Module {1} was successfully installed'
			translated format: {module name})]]
		on: Error do: [:err |
			"Roll back all modules, inform the user and quit"
			modules do: [:module | [module rollBack] on: Error do: ["Nothing"]].
			self inform: err description.
			SmalltalkImage current quitPrimitive].
	modules do: [:module | 
		self register: module name.
		module close].
	"Only save the image after installing all modules"
	WorldState addDeferredUIMessage: [SmalltalkImage current saveSession]
! !

!PEInstaller class methodsFor: 'system startup' stamp: 'Richo 3/14/2012 18:40'!
searchForModules
	^ PEModule defaultDirectory fileNames
		select: [:each | (PEModule isValidName: each)
			and: [(self isAlreadyInstalled: each) not]]
		thenCollect: [:each | PEModule named: each]! !

!PEInstaller class methodsFor: 'system startup' stamp: 'Richo 3/14/2012 17:02'!
startUp: resuming
	resuming ifTrue: [self searchAndInstall]! !

Object subclass: #PEModule
	instanceVariableNames: 'zip file'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-Installer'!

!PEModule class methodsFor: 'constants' stamp: 'Richo 3/14/2012 18:02'!
defaultDirectory
	^ FileDirectory default! !

!PEModule class methodsFor: 'class initialization' stamp: 'Richo 3/14/2012 19:25'!
initialize
	"
	self initialize.
	"
	ExternalDropHandler
		registerHandler: (ExternalDropHandler
			type: nil
			extension: self peModuleExtension
			action: [:file || msg |
				msg := String streamContents: [:stream |
					stream
						nextPutAll: ('It seems you''re trying to install an external module named {1}'
							translated format: {file localName});
						cr;
						cr;
						nextPutAll: ('Please follow the instructions below:
1) Copy the file named {1} into the directory where you have Physical Etoys installed.
2) Close and restart Physical Etoys.
3) The module will be installed automatically.

Thanks for using Physical Etoys!!' translated format: {file localName printString})].
				StringHolder new
					contents: msg;
					openLabel: 'How to install external modules' translated])! !

!PEModule class methodsFor: 'testing' stamp: 'Richo 3/14/2012 18:41'!
isValidName: aByteString 
	^ (FileDirectory extensionFor: aByteString) = self peModuleExtension! !

!PEModule class methodsFor: 'instance creation' stamp: 'Richo 3/14/2012 18:02'!
named: fileName
	(self defaultDirectory fileExists: fileName)
		ifFalse: [^ self error: ('The file {1} doesn''t exists' translated format: {fileName})].
	^ self basicNew initializeWith: fileName! !

!PEModule class methodsFor: 'constants' stamp: 'Richo 3/14/2012 18:41'!
peModuleExtension
	^ 'pem'! !

!PEModule methodsFor: 'initialization' stamp: 'Richo 3/14/2012 18:05'!
close
	file close.
	zip close! !

!PEModule methodsFor: 'accessing' stamp: 'Richo 3/14/2012 18:03'!
defaultDirectory
	^ self class defaultDirectory! !

!PEModule methodsFor: 'private' stamp: 'Richo 3/14/2012 18:23'!
deleteAllDirectoriesExtracted
	"The directories might contain each other, and they can be removed unless they're empty.
	So, keep iterating until nothing gets deleted"
	| deletedFlag |
	[deletedFlag := false.
	self directories do: [:path || dir |
		dir := FileDirectory on: path.
		(dir exists and: [dir entries isEmpty])
			ifTrue: [dir containingDirectory 
				deleteDirectory: dir localName.
				deletedFlag := true]]]
		doWhileTrue: [deletedFlag]! !

!PEModule methodsFor: 'private' stamp: 'Richo 3/14/2012 18:10'!
deleteAllFilesExtracted
	self files do: [:path | 
		self defaultDirectory
			deleteFileNamed: path
			ifAbsent: ["It was deleted already. Do nothing"]].! !

!PEModule methodsFor: 'accessing' stamp: 'Richo 3/14/2012 18:23'!
directories
	^ (zip members 
		select: [:zipMember | zipMember isDirectory])
		collect: [:zipMember |
			self defaultDirectory fullPathFor: zipMember localFileName]! !

!PEModule methodsFor: 'private' stamp: 'Richo 3/14/2012 18:03'!
extractAll
	zip extractAllTo: self defaultDirectory! !

!PEModule methodsFor: 'accessing' stamp: 'Richo 3/14/2012 18:20'!
files
	^ (zip members 
		reject: [:zipMember | zipMember isDirectory])
		collect: [:zipMember | 
			self defaultDirectory fullPathFor: zipMember localFileName]! !

!PEModule methodsFor: 'initialization' stamp: 'Richo 3/14/2012 18:05'!
initializeWith: aString
	file := self defaultDirectory fileNamed: aString.
	zip := ZipArchive new readFrom: file.
	self initialize! !

!PEModule methodsFor: 'public' stamp: 'Richo 3/14/2012 18:56'!
install
	[self extractAll;
		loadSourceFiles]
		on: Error
		do: [:err | 
			self error: ('An error occurred while installing module {1}'
				translated format: {self name}),
				String cr,
				err description]! !

!PEModule methodsFor: 'private' stamp: 'Richo 3/14/2012 18:08'!
loadSourceFile: fileName
	"Assumes the file is source code. It won't work otherwise"
	| fileIn |
	fileIn := [:fn || ff |
		[ff := self defaultDirectory readOnlyFileNamed: fn.
		ff fileIn]
			on: Error do: [:err | ff ifNotNil: [ff close].
				err pass]].
	[(FileDirectory extensionFor: fileName)
		caseOf: {
			['st'] -> [fileIn value: (self defaultDirectory fullNameFor: fileName)].
			['cs'] -> [fileIn value: (self defaultDirectory fullNameFor: fileName)].
			['mcz'] ->[(self defaultDirectory
						fileNamed: fileName
						do: [:mczFile |
							MCMczReader
								loadVersionStream: mczFile
								fromDirectory: self defaultDirectory])]
			}]
		ensure: [self defaultDirectory deleteFileNamed: fileName]! !

!PEModule methodsFor: 'private' stamp: 'Richo 3/14/2012 18:06'!
loadSourceFiles
	(zip memberNames
		select: [:entryName |
			self validSourceTypes includes: (FileDirectory extensionFor: entryName)])
		asSortedCollection
		do: [:entryName | self loadSourceFile: entryName]! !

!PEModule methodsFor: 'accessing' stamp: 'Richo 3/14/2012 18:36'!
name
	^ file localName! !

!PEModule methodsFor: 'public' stamp: 'Richo 3/14/2012 18:45'!
rollBack
	self deleteAllFilesExtracted;
		deleteAllDirectoriesExtracted! !

!PEModule methodsFor: 'accessing' stamp: 'Richo 3/14/2012 18:07'!
validSourceTypes
	^ #('st' 'cs' 'mcz')! !

Object subclass: #PEProject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-Projects'!

!PEProject class methodsFor: 'class initialization' stamp: 'Richo 3/14/2012 19:11'!
initialize
	"
	self initialize.
	"
	ExternalDropHandler
		registerHandler: (ExternalDropHandler
			type: nil
			extension: self peProjectExtension
			action: [:stream | PEProject openProjectStream: stream])
	! !

!PEProject class methodsFor: 'reading/writing' stamp: 'Richo 3/6/2012 18:00'!
openProjectNamed: fileName on: directory
	(directory isKindOf: FileDirectory)
		ifFalse: [^ self].
	directory fileNamed: fileName do: [:file |
		self openProjectStream: file]! !

!PEProject class methodsFor: 'reading/writing' stamp: 'Richo 3/8/2012 17:07'!
openProjectStream: stream
	FileDirectory default temporaryDirectoryDo: [:tempDir || archive |
		archive := ZipArchive new.
		[archive
			readFrom: stream;
			extractAllTo: tempDir]
				ensure: [archive close.
						stream close].
		(archive memberNames
			detect: [:each | each endsWith: '.pr'] ifNone: [nil])
			ifNotNil: [:actualProject |
				ProjectLoading openOn: (tempDir fileNamed: actualProject)]]! !

!PEProject class methodsFor: 'accessing' stamp: 'Richo 3/13/2012 17:52'!
peProjectExtension
	^ 'pe'! !

!PEProject class methodsFor: 'reading/writing' stamp: 'Richo 3/13/2012 17:53'!
storeProjectNamed: projectName in: directory
	"Don't call this for saving a project, instead send:
		
		project storeOnServerAsPE
	"
	ZipArchive new 
		addFile: (directory fullNameFor: projectName) as: projectName;
		writeToFileNamed: (directory fullNameFor: (projectName allButLast: 2), 
							self peProjectExtension);
		close.
	directory deleteFileNamed: projectName ifAbsent:[]! !

Object subclass: #PESerialPort
	instanceVariableNames: 'baudRate portNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-SerialPort'!

PESerialPort class
	instanceVariableNames: 'squeakSerialPort'!

!PESerialPort commentStamp: 'Richo 12/20/2011 12:41' prior: 0!
This is a wrapper for the ScratchPlugin serial port primitives.

It has the same protocol than the SerialPort class, so that the two could be used interchangeably. Only a subset of the full protocol is supported though (the one used by Physical Etoys).

In the class side there are a few extra methods to deal with the serial port list vocabulary used in etoys slots.

This class will replace the use of SerialPort in windows. See class side >> #startUp.!

PESerialPort class
	instanceVariableNames: 'squeakSerialPort'!

!PESerialPort class methodsFor: 'vocabulary' stamp: 'Richo 12/13/2011 16:31'!
availablePorts
	self updateVocabulary. "Just in case"
	^ PESerialPortList availableSerialPorts asSet asSortedCollection, #(#'other...' )! !

!PESerialPort class methodsFor: 'vocabulary' stamp: 'Richo 3/23/2012 13:16'!
availablePortsForNxt
	self updateVocabularyForNxt. "Just in case"
	^ #(#USB), PESerialPortList availableSerialPorts asSet asSortedCollection, #(#'other...' )! !

!PESerialPort class methodsFor: 'class initialization' stamp: 'Richo 12/20/2011 11:47'!
initialize
"
	self initialize.
"
	Smalltalk addToStartUpList: self.
	self startUp! !

!PESerialPort class methodsFor: 'class initialization' stamp: 'Richo 12/20/2011 11:39'!
reinstallSqueakSerialPort
	"If, for some reason, we need to get back the original references to #SerialPort we can reinstall it"
	squeakSerialPort isNil ifTrue: [^ self].
	[Smalltalk at: #SerialPort put: squeakSerialPort]
               on: AttemptToWriteReadOnlyGlobal
               do:[:ex| ex resume: true].
	squeakSerialPort := nil.! !

!PESerialPort class methodsFor: 'class initialization' stamp: 'Richo 12/20/2011 11:38'!
replaceSqueakSerialPort
	"It will replace all references to the global #SerialPort with a reference to #PESerialPort. This allows me to provide a different abstraction for SerialPort than the one squeak provides, but without changing all the code that uses it.
	It's a dirty hack, I know but it shouldn't be dangerous"
	squeakSerialPort notNil ifTrue: [^ self].
	squeakSerialPort := SerialPort.
	[Smalltalk at: #SerialPort put: PESerialPort]
               on: AttemptToWriteReadOnlyGlobal
               do:[:ex| ex resume: true].! !

!PESerialPort class methodsFor: 'system startup' stamp: 'Richo 12/20/2011 11:46'!
startUp
	"If we open this image in windows, replace the squeak serial port. Otherwise, reinstall original SerialPort class"
	SmalltalkImage current platformName = 'Win32'
		ifTrue: [self replaceSqueakSerialPort]
		ifFalse: [self reinstallSqueakSerialPort]! !

!PESerialPort class methodsFor: 'vocabulary' stamp: 'Richo 12/13/2011 16:29'!
updateVocabulary
	Vocabulary addStandardVocabulary: (PluggableListType new vocabularyName: #PEPort;
		 block: [self availablePorts]).
! !

!PESerialPort class methodsFor: 'vocabulary' stamp: 'Richo 3/23/2012 13:15'!
updateVocabularyForNxt
	Vocabulary addStandardVocabulary: (PluggableListType new vocabularyName: #PENxtPort;
		 block: [self availablePortsForNxt]).
! !

!PESerialPort methodsFor: 'settings' stamp: 'Richo 12/20/2011 12:37'!
baudRate
	^ baudRate! !

!PESerialPort methodsFor: 'settings' stamp: 'Richo 12/20/2011 12:10'!
baudRate: aSmallInteger 
	baudRate := aSmallInteger! !

!PESerialPort methodsFor: 'open/close' stamp: 'Richo 12/20/2011 12:15'!
close
	portNumber isNil ifTrue: [^ self].
	self plugin closePort: portNumber.
	portNumber := nil! !

!PESerialPort methodsFor: 'initialize-release' stamp: 'Richo 12/20/2011 12:20'!
initialize
	super initialize.
	baudRate := 9600! !

!PESerialPort methodsFor: 'input/output' stamp: 'Richo 12/20/2011 12:16'!
nextPutAll: aByteArray 
	portNumber isNil ifTrue: [^ self error: 'The serial port must be open'].
	^ self plugin writePort: portNumber data: aByteArray! !

!PESerialPort methodsFor: 'open/close' stamp: 'Richo 12/20/2011 12:17'!
openPort: portId
	portNumber notNil ifTrue: [^ self].
	portNumber := portId isString
		ifTrue: [self plugin openPortNamed: portId baud: baudRate]
		ifFalse: [self plugin openPortNamed: 'COM', portId asString baud: baudRate].
	portNumber = -1
		ifTrue: [^ portNumber := nil]
		ifFalse: [^ self]! !

!PESerialPort methodsFor: 'accessing' stamp: 'Richo 12/20/2011 12:10'!
plugin
	^ ScratchPlugin! !

!PESerialPort methodsFor: 'input/output' stamp: 'Richo 12/23/2011 19:15'!
readByteArray
	| buffer bytesRead |
	portNumber isNil ifTrue: [^ self].
	bytesRead := self plugin readPort: portNumber into: (buffer := ByteArray new: 1024).
	^ buffer allButLast: (1024 - bytesRead)
	! !

!PESerialPort methodsFor: 'input/output' stamp: 'Richo 12/22/2011 16:09'!
readString
	^ self readByteArray asString! !

Object subclass: #PESerialPortList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-SerialPort'!

!PESerialPortList commentStamp: 'Richo 3/9/2010 10:21' prior: 0!
I provide a general facility to list serial ports in windows and unix.!

!PESerialPortList class methodsFor: 'mac' stamp: 'TS 9/3/2010 01:13'!
availableBluetoothPortsForMac
	| pattern1 |
	pattern1 := 'cu.Blu*'.
	 ^(FileDirectory on: '/dev') fileNames
		select: [:e | (pattern1 match: e)]
		thenCollect: [:e |( '/dev/' , e)  asSymbol]! !

!PESerialPortList class methodsFor: 'general' stamp: 'Richo 6/8/2011 17:19'!
availableSerialPorts
	^self platformName caseOf: {
		['unix'] -> [self availableSerialPortsForUnix].
		['Mac OS'] -> [self availableSerialPortsForMac].
		['Win32'] -> [self availableSerialPortsForWindows]
	} otherwise: [self error: 'Not supported platform' translated]! !

!PESerialPortList class methodsFor: 'mac' stamp: 'TS 9/3/2010 01:11'!
availableSerialPortsForMac

	| pattern1 |
	pattern1 := 'cu.*'.
	 ^(FileDirectory on: '/dev') fileNames
		select: [:e | (pattern1 match: e)]
		thenCollect: [:e |( '/dev/' , e)  asSymbol]! !

!PESerialPortList class methodsFor: 'unix' stamp: 'Richo 6/8/2011 17:22'!
availableSerialPortsForUnix
	| patterns |
	patterns := #('ttyS*' 'rfcomm*' 'ttyUSB*').
	 ^(FileDirectory on: '/dev') fileNames
		select: [:e | (patterns anySatisfy: [:pattern | pattern match: e])]
		thenCollect: [:e |( '/dev/' , e)  asSymbol]! !

!PESerialPortList class methodsFor: 'win32' stamp: 'Richo 1/6/2012 18:12'!
availableSerialPortsForWindows
	"It requires ProcessWrapper to run a console application called ComEnumerator. The executable ComEnumerator.exe must be in the same directory as the image."
	^ [(FileDirectory default fileExists: 'ComEnumerator.exe')
			ifFalse: [self error: 'ComEnumerator not found'].
		(((Smalltalk at: #ProcessWrapper) new
		    useStdout;
		    startWithShellCommand: 'ComEnumerator';
		    upToEnd) substrings collect: [:each | each asSymbol])
			ifEmpty: [self error: 'No available serial port was found']
			ifNotEmpty: [:col | col]]
		on: Error do: [(1 to: 9) collect: [:n | ('COM' , n asString) asSymbol]]! !

!PESerialPortList class methodsFor: 'mac' stamp: 'TS 9/3/2010 01:18'!
availableUSBPortsForMac

	| pattern1 |
	pattern1 := 'cu.usb*'.
	 ^(FileDirectory on: '/dev') fileNames
		select: [:e | (pattern1 match: e)]
		thenCollect: [:e |( '/dev/' , e)  asSymbol]! !

!PESerialPortList class methodsFor: 'accessing' stamp: 'Richo 11/10/2009 16:13'!
platformName
^ SmalltalkImage current platformName! !

!PESerialPortList class methodsFor: 'accessing' stamp: 'Richo 6/16/2011 15:45'!
watcher
	"Returns an object that continuously updates its value with the current available serial ports list.

		self watcher openInHand.
	"
	^ (UpdatingStringMorph 
		on: [PESerialPortList availableSerialPorts asArray]
		selector: #value)
		stepTime: 1000! !

Object subclass: #PETranslator
	instanceVariableNames: 'translations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-Utilities'!

PETranslator class
	instanceVariableNames: 'translators'!

PETranslator class
	instanceVariableNames: 'translators'!

!PETranslator class methodsFor: 'finding strings' stamp: 'Richo 9/9/2011 18:44'!
allTranslatedCallsFrom: class
	"This looks in the class itself and in its metaclass"
	^ (SystemNavigation default allCallsOn: #translated from: class),
		(SystemNavigation default allCallsOn: #translatedNoop from: class),
		(SystemNavigation default allCallsOn: #translated from: class class),
		(SystemNavigation default allCallsOn: #translatedNoop from: class class)! !

!PETranslator class methodsFor: 'creating files' stamp: 'Richo 9/9/2011 16:31'!
createTranslationFileFor: aDictionary in: aLocaleID
	| file |
	file := (self directoryForLocaleID: aLocaleID)
				forceNewFileNamed: self translationsFileName.
	[file nextPutAll: (aDictionary asCSVStringWithDelimiter: $;)]
		ensure: [file close]! !

!PETranslator class methodsFor: 'creating files' stamp: 'Richo 9/9/2011 19:40'!
createTranslationFiles
	| strings dict translate |
	translate := [:string :aLocaleID || translation |
		TextDomainManager allKnownDomains
			detect: [:domain | 
				translation := string translatedTo: aLocaleID inDomain: domain.
				string ~= translation]
			ifNone: [translation := string].
		translation].
	strings := self findAllStrings asSet.
	NaturalLanguageTranslator availableLanguageLocaleIDs
		do: [:localeID |
		dict := Dictionary new.
		strings do: [:str |
			dict at: str put: (translate value: str value: localeID)].
		self createTranslationFileFor: dict in: localeID
		]! !

!PETranslator class methodsFor: 'instance creation' stamp: 'Richo 9/9/2011 16:15'!
current
	^ self for: LocaleID current! !

!PETranslator class methodsFor: 'accessing' stamp: 'Richo 9/9/2011 16:36'!
directoryForLocaleID: aLocaleID
	"
	locale/es/
	"
	^ self
		directoryNamed: aLocaleID posixName
		in: (self directoryNamed: 'locale' in: FileDirectory default)! !

!PETranslator class methodsFor: 'accessing' stamp: 'Richo 9/9/2011 15:44'!
directoryNamed: dirName in: directory
	(directory directoryExists: dirName)
		ifFalse: [directory createDirectory: dirName].
	^ directory directoryNamed: dirName! !

!PETranslator class methodsFor: 'finding strings' stamp: 'Richo 9/8/2011 16:19'!
findAllStrings
	^ Array streamContents: [:stream |
		PEUtilities packages collect: [:each |
			stream nextPutAll: (self findAllStringsInPackage: each)
		]]! !

!PETranslator class methodsFor: 'finding strings' stamp: 'Richo 9/9/2011 18:46'!
findAllStringsInPackage: packageName 
	^ Array streamContents: [:stream |
		(PackageInfo named: packageName) classes
			do: [:class | 
				"Look if it has additions to viewer categories"
				stream nextPutAll: (self viewerCategoriesStringsFrom: class).
				"Look for all the #translated/#translatedNoop calls"
				(self allTranslatedCallsFrom: class)
					do: [:method |
						(RBParser parseMethod: method sourceString)
							nodesDo: [:each | 
								(each isMessage
									and: [each receiver isLiteral
									and: [#(translated translatedNoop) includes: each selector]])
										ifTrue: [stream nextPut: each receiver value]]]]]! !

!PETranslator class methodsFor: 'instance creation' stamp: 'Richo 9/9/2011 17:23'!
for: localeID
	^ self translators
		at: localeID
		ifAbsentPut: [self basicNew initializeWith: localeID]! !

!PETranslator class methodsFor: 'class initialization' stamp: 'Richo 9/12/2011 12:44'!
initialize
	"
	self initialize
	"
	Smalltalk addToShutDownList: self.
	self resetTranslators! !

!PETranslator class methodsFor: 'class initialization' stamp: 'Richo 9/9/2011 17:23'!
resetTranslators
	translators := nil! !

!PETranslator class methodsFor: 'system startup' stamp: 'Richo 9/12/2011 12:44'!
shutDown
	self resetTranslators! !

!PETranslator class methodsFor: 'translating' stamp: 'Richo 9/9/2011 16:06'!
translate: aString to: localeID ifError: aBlock
	^ (self for: localeID)
			translate: aString
			ifError: aBlock! !

!PETranslator class methodsFor: 'accessing' stamp: 'Richo 9/9/2011 15:39'!
translationsFileName
	^ 'pe.strings'! !

!PETranslator class methodsFor: 'accessing' stamp: 'Richo 9/9/2011 17:22'!
translators
	^ translators ifNil: [translators := Dictionary new]! !

!PETranslator class methodsFor: 'finding strings' stamp: 'Richo 9/12/2011 09:58'!
viewerCategoriesStringsFrom: class
	| strings |
	strings := OrderedCollection new.
	class class selectors
		select: [:selector |'additionsToViewer*' match: selector] 
		thenDo: [:selector || categories |
			categories := (selector beginsWith: #additionsToViewerCategories)
				ifTrue: [class perform: selector]
				ifFalse: [{class perform: selector}].
			categories
				do: [:cat |
					strings add: cat first fromCamelCase. "Category name"
					cat second
						do: [:slotOrCommand || tileName tileComment |
							tileName := slotOrCommand second fromCamelCase.
							tileComment := slotOrCommand third.
							(tileName includes: $:) ifTrue: [tileName := tileName copyWithout: $:].
							strings add: tileName;
								add: tileComment
						]	
					]].
	^ strings asArray! !

!PETranslator methodsFor: 'initialize-release' stamp: 'Richo 4/4/2012 15:59'!
initializeWith: aLocaleID
	| parser |
	self initialize.
	translations := Dictionary new.
	parser := Smalltalk at: #CSVParser ifAbsent: [^ self].
	(self class directoryForLocaleID: aLocaleID)
		fileNamed: self class translationsFileName
		do: [:file |
			(parser parse: file delimiter: $;)
				do: [:row | translations at: row first put: row second]]! !

!PETranslator methodsFor: 'translating' stamp: 'Richo 9/9/2011 16:07'!
translate: aString ifError: aBlock
	^ translations at: aString ifAbsent: aBlock! !

Object subclass: #PEUtilities
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-Utilities'!

PEUtilities class
	instanceVariableNames: 'baseRepositories physicalEtoysForm peRootProject'!

!PEUtilities commentStamp: 'Richo 3/9/2010 10:18' prior: 0!
I'm just an utility class for stuff related to Physical Etoys. I exist only for convenience.!

PEUtilities class
	instanceVariableNames: 'baseRepositories physicalEtoysForm peRootProject'!

!PEUtilities class methodsFor: 'about window' stamp: 'Richo 4/7/2011 14:13'!
aboutWindowLabel
	^ 'About Physical Etoys'! !

!PEUtilities class methodsFor: 'info' stamp: 'Richo 4/3/2012 14:51'!
authors
	"List of the author's names to appear in the about window"
	^ #('Lic. Gonzalo Zabala' 'Ricardo Moran' 'Sebastin Blanco' 'Matas Teragni')! !

!PEUtilities class methodsFor: 'class initialization' stamp: 'Richo 4/3/2012 14:42'!
baseRepositories
	^ baseRepositories ! !

!PEUtilities class methodsFor: 'releasing' stamp: 'Richo 4/3/2012 12:50'!
changeSugarHelpButtonToOpenAboutWindow
"This method modifies the help button in the Sugar Navigator Bar to open Physical Etoys' about window.
DANGER: This method is destructive!! It modifies the source code. It should only be used to release Physical Etoys
"
	(Smalltalk
		at: #SugarNavigatorBar
		ifAbsent: [^ self])
		compile: 'toggleHelp
	"This method was autogenerated by the Physical Etoys release process"
	PEUtilities openAboutWindow'! !

!PEUtilities class methodsFor: 'class initialization' stamp: 'Richo 9/8/2011 16:06'!
compileSqNumberParserIfNotFound
	"Compile an empty SqNumberParser class that delegates to Number if the class is not found in the system. This allows us to use RBParser in Etoys"
	
	(Smalltalk hasClassNamed: #SqNumberParser)
		ifFalse: [| newParser |
			newParser := Object subclass: #SqNumberParser
				instanceVariableNames: ''
				classVariableNames: ''
				poolDictionaries: ''
				category: 'Richo'.
			newParser class compile: 'parse: aString ^ Number readFrom: aString']! !

!PEUtilities class methodsFor: 'info' stamp: 'Richo 4/3/2012 14:51'!
currentVersion
	^ '2.0'! !

!PEUtilities class methodsFor: 'releasing' stamp: 'Richo 1/6/2011 13:16'!
deleteQuickGuides
"
This method will remove all the QuickGuides installed in the locale directory. Since the Quickguides won't be used by Physical Etoys, this will save disk space.
"
| locale |
locale := FileDirectory default directoryNamed: 'locale'.
locale directoryNames do: [:d1 | | country |
	country := locale directoryNamed: d1.
	(country directoryExists: 'QuickGuides')
			ifTrue: [
(country directoryNamed: 'QuickGuides') deleteLocalFiles.
country deleteDirectory: 'QuickGuides'
	]
]! !

!PEUtilities class methodsFor: 'mime encoding' stamp: 'Richo 11/17/2009 12:57'!
encodeFileNamed: fileName
^(Base64MimeConverter mimeEncode: ((FileStream readOnlyFileNamed: fileName) binary)) upToEnd! !

!PEUtilities class methodsFor: 'mime encoding' stamp: 'Richo 11/17/2009 12:58'!
formFromEncodedPNG: aString
^(PNGReadWriter on: (Base64MimeConverter mimeDecodeToBytes: aString readStream)) nextImage.! !

!PEUtilities class methodsFor: 'cleaning' stamp: 'Richo 4/13/2011 11:27'!
garbageCollect
	| tasks |
	tasks := OrderedCollection new
				add: [Player freeUnreferencedSubclasses];				
				add: [MCFileBasedRepository flushAllCaches];				
				add: [SMSqueakMap default clearCaches];				
				add: [Smalltalk removeEmptyMessageCategories];				
				add: [Workspace
						allSubInstancesDo: [:each | each setBindings: Dictionary new]];				
				add: [Undeclared removeUnreferencedKeys];				
				add: [Categorizer sortAllCategories];				
				add: [Symbol compactSymbolTable];				
				add: [ReleaseBuilder new fixObsoleteReferences];				
				add: [Smalltalk garbageCollectMost];
				 yourself.
	Utilities
		informUserDuring: [:bar | tasks
				do: [:block | 
					bar value: block printString.
					[block value]
						on: Error
						do: [:error | Transcript show: error;
								 cr]]].
	^ SystemNavigation default obsoleteClasses! !

!PEUtilities class methodsFor: 'class initialization' stamp: 'Richo 4/3/2012 14:36'!
initialize
	"
		self initialize.
	"
	self compileSqNumberParserIfNotFound;
		storeBaseRepositories.! !

!PEUtilities class methodsFor: 'installer' stamp: 'Richo 4/12/2011 18:07'!
installerScript
"
	Transcript open; show: self installerScript.
"

	^ String streamContents: [:stream |
		self packageVersions do: [:package |
			stream nextPutAll: ('(Installer ss project: {1}) install: {2}.' format: {(self projectFor: package) printString. package printString}); cr.
		]
	]	
! !

!PEUtilities class methodsFor: 'root project testing' stamp: 'Richo 4/10/2012 16:37'!
isEtoysProjectClean: aProject
	"It tries to ensure this project can be saved in the squeakland showcase."
	"Checks if the given project contains any morph instance of a class
	belonging to the Physical Etoys packages."
	"It's probably not the best way of testing, but I don't know any better
	for now"
	| packages morphClasses |
	packages := PEUtilities packages collect: [:each |
		PackageInfo named: each].
	morphClasses := packages inject: #() into: [:subTotal :next |
		subTotal, 
		(next classes select: [:each | each inheritsFrom: Morph])].
	aProject world allMorphsDo: [:morph |
		(morphClasses 
			detect: [:each | morph isKindOf: each] 
			ifNone: [nil])
			ifNotNil: [^ false]].
	^ true
! !

!PEUtilities class methodsFor: 'root project testing' stamp: 'Richo 4/10/2012 15:59'!
isPEProject: aProject
	"Traverses the project's hierarchy looking for the physical etoys
	root project (which must be set manually)."
	| currentProject |
	currentProject := aProject.
	[(currentProject notNil and: [currentProject = self peRootProject])
		ifTrue: [^ true].
	currentProject := currentProject parent]
		doWhileFalse: [currentProject isNil or: [currentProject isTopProject]].
	"The physical etoys root project was not found"
	^ false! !

!PEUtilities class methodsFor: 'info' stamp: 'Richo 4/12/2011 13:24'!
license
	^ 'All images are licensed under CC Attribution-ShareALike (http://creativecommons.org/licenses/by-sa/3.0/). The Arduino images were created with Fritzing (http://fritzing.org/). The source code is licensed under the MIT License with parts under the Apache License.

Copyright (c) 1996-2010 Viewpoints Research Institute, and Contributors

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

You may obtain a copy of the Apache License at http://www.apache.org/licenses/LICENSE-2.0

Copyright (c) 1996 Apple Computer, Inc. All Rights Reserved.'! !

!PEUtilities class methodsFor: 'flap creation' stamp: 'Richo 11/17/2009 12:43'!
makeNewFlapNamed: flapName with: anArray 
	"Example:
	PEUtilities makeNewFlapNamed: 'A flap' with: 
	#(
	(RectangleMorph newStandAlone 'A rectangle' '')
	(CircleMorph newStandAlone 'A circle' '')
	)
	"
	| aStrip aFlapTab |
	aStrip := PartsBin
				newPartsBinWithOrientation: #leftToRight
				andColor: (Color blue muchLighter alpha: 0.8)
				from: anArray
				buttonClass: IconicButton.
	aFlapTab := FlapTab new referent: aStrip beSticky.
	aFlapTab
		setName: flapName translated
		edge: #bottom
		color: Color blue muchLighter.
	aFlapTab setBalloonText: aFlapTab balloonTextForFlapsMenu.
	aStrip beFlap: true.
	aStrip extent: self currentWorld width @ 78.
	aStrip
		submorphsDo: [:ea | ea updateImage].
	aFlapTab openInHand! !

!PEUtilities class methodsFor: 'flap creation' stamp: 'Richo 4/13/2011 13:30'!
makePESuppliesFlap
	"
	self makePESuppliesFlap.
	"
	| aFlapTab anObjectsTool |
	anObjectsTool := PESupplies new.
	aFlapTab := SugarSuppliesTab new referent: anObjectsTool beSticky.
	aFlapTab setName: 'Physical Etoys' translated edge: #bottom color: Color gray lighter.
	aFlapTab position: Display width - (aFlapTab width + 22) @ 0.
	aFlapTab setBalloonText: aFlapTab balloonTextForFlapsMenu.
	anObjectsTool extent: self currentWorld width @ 200.
	anObjectsTool beFlap: true.
	anObjectsTool color: Color blue muchLighter.
	anObjectsTool clipSubmorphs: true.
	aFlapTab arrangeToPopOutOnMouseOver: true;
		 useSolidTab;
		 height: 20;
		 openInWorld.
	Flaps addGlobalFlap: aFlapTab.
	self currentWorld addGlobalFlaps! !

!PEUtilities class methodsFor: 'releasing' stamp: 'Richo 4/3/2012 14:51'!
makePhysicalEtoysRelease
"
	This method will configure the image as a Physical Etoys image, 
	and it will save it (make sure to have a backup copy at hand in 
	case the process goes wrong).
	
	Important: *Run this method in the Main project (the one you 
	want to appear first when Physical Etoys is opened)*
"
"
	self makePhysicalEtoysRelease.
"
self writeReadmeFile. "Write the README.txt file"
self writeSplashFile. "Write the splash.bmp file"
self changeSugarHelpButtonToOpenAboutWindow. "Instead of opening quickguides open about window"
self deleteQuickGuides. "Delete the QuickGuides file to make some space"
Project home "Rename home project to avoid open it at startup (see SugarLauncher>>welcome:)"
	ifNotNil: [:p | p renameTo: 'Etoys-Home'].
Project current setParent: Project current. "Make this project the top project"
Preferences setPreference: #useLocale toValue: true. "The language will be configured by default"
Preferences setPreference: #eToyFriendly toValue: true. "The user won't be able to save the image"
Preferences setPreference: #warnIfNoChangesFile toValue: false. "We don't ship the changes file"
Preferences setPreference: #warnIfNoSourcesFile toValue: false. "We don't ship the sources file"
self garbageCollect. "Clean memory of unused objects"
self prepareMainUI. "Prepare UI"
1 second asDelay wait. "A delay, just in case"
SmalltalkImage current snapshot: true andQuit: false. "Save the image and quit"! !

!PEUtilities class methodsFor: 'about window' stamp: 'Richo 4/7/2011 14:10'!
newAboutWindow
	| aboutWindow |
	aboutWindow := ((AlignmentMorph inAColumn: {
		self physicalEtoysLogo.
		self projectInfo.
		self uaiLogo
	}) openInWindowLabeled: self aboutWindowLabel)
		setWindowColor: (Color
				r: 0.516
				g: 0.806
				b: 1.0);
		color: Color white.
	^ aboutWindow extent: aboutWindow extent + (60@10)! !

!PEUtilities class methodsFor: 'about window' stamp: 'Richo 4/7/2011 13:23'!
openAboutWindow
	| openedWindows aboutWindow |
	openedWindows := SystemWindow
				windowsIn: World
				satisfying: [:w | w label = self aboutWindowLabel].
	aboutWindow := openedWindows isEmpty
				ifTrue: [self newAboutWindow]
				ifFalse: [openedWindows first].
	aboutWindow expand comeToFront ! !

!PEUtilities class methodsFor: 'info' stamp: 'Richo 4/3/2012 14:54'!
packageVersions
	"Asks monticello for each package version"
	^ self packages collect: [:pckgName || wc version repo |
		wc := (MCPackage named: pckgName) workingCopy.
		wc ancestry ancestorString
			ifEmpty: [pckgName] 
			ifNotEmpty: [:pckgVersion | pckgVersion]]! !

!PEUtilities class methodsFor: 'info' stamp: 'Richo 4/3/2012 14:50'!
packages
	"
	self packages
	"
	^ ((PackageInfo allPackages
		reject: [:pkgInfo || pkg wc |
			pkg := MCPackage named: pkgInfo packageName.
			wc := pkg workingCopy.
			baseRepositories includesAllOf: wc repositoryGroup repositories])
		collect: [:each | each name])
			asSortedCollection
	! !

!PEUtilities class methodsFor: 'root project testing' stamp: 'Richo 4/10/2012 15:55'!
peRootProject
	^ peRootProject! !

!PEUtilities class methodsFor: 'root project testing' stamp: 'Richo 4/10/2012 15:56'!
peRootProject: aProject 
	peRootProject := aProject! !

!PEUtilities class methodsFor: 'about window' stamp: 'Richo 4/7/2011 14:06'!
physicalEtoysLogo
"
physicalEtoysForm := nil.
"

	^ SketchMorph withForm: (physicalEtoysForm ifNil: [physicalEtoysForm := self formFromEncodedPNG: 'iVBORw0KGgoAAAANSUhEUgAAAT8AAAE1CAYAAACRPefNAAAABHNCSVQFBQUBSsjp7wABFmpJ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'])! !

!PEUtilities class methodsFor: 'releasing' stamp: 'Richo 4/10/2012 17:45'!
prepareMainUI
	"Change display mode to 'scaled to fit'"
	 OLPCVirtualScreen install.
	"Delete all system windows"
	World submorphsDo: [:morph |
		"(morph isFlapTab and: [morph isGlobalFlap])
			ifFalse: [morph delete]"
		morph isSystemWindow ifTrue: [morph delete]].
	"Create the PE Supplies flap"
	self makePESuppliesFlap. 
! !

!PEUtilities class methodsFor: 'installer' stamp: 'Richo 4/13/2011 13:26'!
projectFor: packageName
	"Richo: This method is needed because I'm stupid enough to name the repositories differently than the packages. Luckily I learned from my mistakes and the projects below are the only ones affected"
	| projectName |
	projectName := packageName upTo: $-.
	^ projectName caseOf: {
		['Wiimote'] -> ['SqueakWiimote'].
		['RoboSapien'] -> ['Sapien'].
	} otherwise: [projectName]! !

!PEUtilities class methodsFor: 'about window' stamp: 'Richo 4/13/2011 13:28'!
projectInfo
	^ ('
Authors: {1}

Version: {2}

Website: {3}

' translated format: {
	self authors inject: '' into: [:subTotal :next |  subTotal, String cr, '	* ', next].
	self currentVersion.
	self website
}) asTextMorph lock backgroundColor: Color white
! !

!PEUtilities class methodsFor: 'sunit' stamp: 'Richo 4/6/2011 08:45'!
runAllTestsInCategory: aCategoryName
"Finds all the TestCase subclasses inside the given category and runs all tests. Returns an array of associations where the key is the class and the value is the test results"
^ (SystemOrganization listAtCategoryNamed: aCategoryName) 
	collect: [:className | | class |
		class := Smalltalk at: className.
		(class inheritsFrom: TestCase) 
			ifTrue: [class -> class suite run]]
	thenSelect: [:each | each notNil]! !

!PEUtilities class methodsFor: 'syntax error' stamp: 'Richo 3/30/2012 19:51'!
showSyntaxError: errorMessage translatedCode: translatedCode originalCode: originalCode
	StringHolder new
		contents: (String streamContents: [:stream |
			stream
				nextPutAll: 'There seems to be a syntax error in the script you tried to compile. This indicates there might be a bug in the code generator.' translated;
				cr;
				nextPutAll: 'Please send this report to the developers of the software. Thank you.' translated;
				cr; cr; cr;
				nextPutAll: 'Errors:' translated; cr;
				nextPutAll: '*******'; cr;
				nextPutAll: errorMessage;
				cr; cr; cr;
				nextPutAll: 'Translated source code:' translated; cr;
				nextPutAll: '***********************'; cr;
				nextPutAll: translatedCode;
				cr; cr; cr;
				nextPutAll: 'Original source code:' translated; cr;
				nextPutAll: '********************'; cr;
				nextPutAll: originalCode]);
		openLabel: 'Syntax Error'! !

!PEUtilities class methodsFor: 'class initialization' stamp: 'Richo 4/3/2012 14:47'!
storeBaseRepositories
	"From now on, every repository added will be considered a part of Physical Etoys"
	baseRepositories := Set new.
	PackageInfo allPackages
		collect: [:pkgInfo | MCPackage named: pkgInfo packageName]
		thenDo: [:pkg |
			pkg name = 'PhysicalEtoys'
				ifFalse: [baseRepositories addAll: pkg workingCopy repositoryGroup repositories]]! !

!PEUtilities class methodsFor: 'about window' stamp: 'Richo 4/7/2011 14:01'!
uaiLogo
^ SketchMorph withForm: (self formFromEncodedPNG: 'iVBORw0KGgoAAAANSUhEUgAAAJoAAAAsCAIAAAASDjYoAAAAAXNSR0IArs4c6QAAAARnQU1B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')! !

!PEUtilities class methodsFor: 'info' stamp: 'Richo 4/3/2012 14:51'!
website
	^ 'http://tecnodacta.com.ar/gira'! !

!PEUtilities class methodsFor: 'releasing' stamp: 'Richo 4/12/2011 13:24'!
writeReadmeFile
FileStream forceNewFileNamed: 'README.txt' do: [:file | | contents |
contents := 'Physical Etoys
**************
{1}
Centro de Altos Estudios en Tecnologa Informtica
Universidad Abierta Interamericana
{2}


Description:
------------
Physical Etoys is a visual programming tool that connects the virtual world of computers with the real world in which we live in. With Physical Etoys you can easily program real world objects (such as robots) to perform interesting tasks, or you can sense the world and use that information to control virtual objects (such as drawings on the screen).
Everything is just as easy as grabbing some tiles representing instructions and assembling a script. No need for programming skills, just use the mouse and explore the system and its possibilities!!

Physical Etoys is actually an "extension" to Etoys (http://www.squeakland.org), a media-rich authoring environment and visual programming system made by the very same people who created Smalltalk. We believe Etoys is a wonderful software that helps children explore their own creativity in fun and educational ways, so we wanted to give Etoys the possibility to interact with real world objects such as robotic kits and innovative joysticks. Feel free to learn more about Etoys, it''s open, it''s free, and it''s really fun!!


Installed packages (from SqueakSource):
---------------------------------------
{3}

License:
--------
{4}'
format: {String streamContents: [:str | self authors do: [:author | str nextPutAll: author] separatedBy: [str nextPutAll: ' - ']].
	self website.
	self packageVersions inject: '' into: [:subTotal :next | subTotal, '* ', next, String cr].
	self license}.
file nextPutAll: contents]! !

!PEUtilities class methodsFor: 'releasing' stamp: 'Richo 4/3/2012 17:24'!
writeSplashFile
	| form font string fileName |
	form := self physicalEtoysLogo form deepCopy.
	font := Preferences standardListFont.
	string := 'version ' , self currentVersion.
	form getCanvas
		drawString: string
		at: form boundingBox bottomRight - 2 - ((font widthOfString: string) @ font height)
		font: font
		color: Color gray;
		frameRectangle: form boundingBox
		width: 1
		color: Color black.
	fileName := 'splash.bmp'.
	(FileDirectory default fileExists: fileName)
		ifTrue: [FileDirectory default deleteFileNamed: fileName].
	form writeBMPfileNamed: fileName! !

Object subclass: #PoCompiler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-Gettext'!

!PoCompiler class methodsFor: 'as yet unclassified' stamp: 'Richo 4/18/2011 13:28'!
compilePoDir
"
	self compilePoDir.
"
| localeDir poDir osProcess |
poDir := FileDirectory default directoryNamed: 'po'.
(FileDirectory default directoryExists: 'locale') ifFalse: [FileDirectory default createDirectory: 'locale'].
localeDir := FileDirectory default directoryNamed: 'locale'.
osProcess := Smalltalk at: #OSProcess ifAbsent: [^ self error: 'OSProcess is missing'].

'Compiling po files'
    displayProgressAt: Sensor cursorPoint
    from: 1 to: poDir directoryNames size
    during: [:bar | 
    (poDir directoryNames
	collect: [:dirName | poDir directoryNamed: dirName])
	withIndexDo: [:langDir :index | | langID |
		bar value: index.
		langID := langDir localName.
		langDir fileNames
			select: [:each | '*.po' match: each]
			thenDo: [:fileName |
				(localeDir directoryExists: langID)
					ifFalse: [localeDir createDirectory: langID.
						(localeDir directoryNamed: langID) createDirectory: 'LC_MESSAGES'].
				osProcess waitForCommand: 'msgfmt ', langDir pathName, '/', fileName, ' -o ', localeDir pathName , '/', langID,'/LC_MESSAGES/',  (fileName upTo: $.), '.mo -v'.
				]
		]].
 ! !

!SymbolListTile methodsFor: '*PhysicalEtoys' stamp: 'Richo 3/19/2012 14:06'!
showOptions
	"Overriden to permit Physical Etoys update the serial ports list"
	dataType notNil ifTrue: [self updateChoices].
	super showOptions! !

Player subclass: #FileListPlayer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-FileList'!

!FileListPlayer class methodsFor: 'change set logging' stamp: 'Richo 7/28/2010 13:48'!
isUniClass
	^ self ~~ self officialClass! !

!FileListPlayer class methodsFor: 'change set logging' stamp: 'Richo 7/28/2010 13:48'!
officialClass
	^ FileListPlayer ! !

!FileListPlayer class methodsFor: 'change set logging' stamp: 'Richo 7/28/2010 13:47'!
wantsChangeSetLogging
	^ (self == self officialClass
			or: [(self name beginsWith: self officialClass asString) not])
		or: [Preferences universalTiles]! !

!FileListPlayer methodsFor: 'slots' stamp: 'Richo 7/22/2010 17:06'!
getEntriesCount
^self costume renderedMorph entriesCount! !

!FileListPlayer methodsFor: 'slots' stamp: 'Richo 7/22/2010 20:58'!
getEntryExtension
^self costume renderedMorph selectedEntryExtension! !

!FileListPlayer methodsFor: 'slots' stamp: 'Richo 7/22/2010 16:57'!
getEntryIndex
^self costume renderedMorph selectedEntryIndex! !

!FileListPlayer methodsFor: 'slots' stamp: 'Richo 7/22/2010 16:38'!
getEntryName
^self costume renderedMorph selectedEntryName! !

!FileListPlayer methodsFor: 'slots' stamp: 'Richo 7/22/2010 21:08'!
getEntryPath
^self costume renderedMorph selectedEntryPath ! !

!FileListPlayer methodsFor: 'slots' stamp: 'Richo 7/25/2010 16:41'!
getFilter
^self costume renderedMorph filter! !

!FileListPlayer methodsFor: 'slots' stamp: 'Richo 7/22/2010 17:01'!
getIsDirectory
^self costume renderedMorph selectedEntryIsDirectory! !

!FileListPlayer methodsFor: 'slots' stamp: 'Richo 7/25/2010 16:40'!
getShowDirectories
^self costume renderedMorph showDirectories! !

!FileListPlayer methodsFor: 'commands' stamp: 'Richo 7/22/2010 17:19'!
openDirectory
self costume renderedMorph openDirectory! !

!FileListPlayer methodsFor: 'slots' stamp: 'Richo 7/25/2010 18:15'!
setEntryIndex: aNumber
self getEntriesCount = 0 ifTrue: [^self].
self costume renderedMorph selectedEntryIndex: aNumber - 1 \\ self getEntriesCount + 1! !

!FileListPlayer methodsFor: 'slots' stamp: 'Richo 7/25/2010 16:41'!
setFilter: aString
^self costume renderedMorph filter: aString! !

!FileListPlayer methodsFor: 'slots' stamp: 'Richo 7/25/2010 16:40'!
setShowDirectories: aBoolean
^self costume renderedMorph showDirectories: aBoolean! !

!FileListPlayer methodsFor: 'commands' stamp: 'Richo 7/25/2010 16:40'!
updateCurrentDirectory
self costume renderedMorph updateCurrentDirectory! !

!Player methodsFor: '*physicaletoys-compilersupport' stamp: 'Richo 1/27/2011 15:15'!
isUsingCompiledMode
	^ false! !

!Player methodsFor: '*physicaletoys-compilersupport' stamp: 'Richo 2/29/2012 13:29'!
updateScriptEditors
	"Do nothing. Subclasses should override this if necessary."! !

Player subclass: #SpreadsheetPlayer
	instanceVariableNames: 'rowIndex columnIndex fileName separator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-Spreadsheet'!

!SpreadsheetPlayer class methodsFor: 'change set logging' stamp: 'Richo 7/9/2010 15:14'!
isUniClass
	^ self ~~ self officialClass! !

!SpreadsheetPlayer class methodsFor: 'change set logging' stamp: 'Richo 7/9/2010 15:15'!
officialClass
	^ SpreadsheetPlayer! !

!SpreadsheetPlayer class methodsFor: 'change set logging' stamp: 'Richo 7/9/2010 15:15'!
wantsChangeSetLogging
	^ (self == self officialClass
			or: [(self name beginsWith: self officialClass asString) not])
		or: [Preferences universalTiles]! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/29/2010 00:41'!
clearColumn
[^self costume renderedMorph clearColumn: self getColumnIndex] on: Error do: [:err| self inform: err description]! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/29/2010 00:41'!
clearRow
[^self costume renderedMorph clearRow: self getRowIndex] on: Error do: [:err| self inform: err description]! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/29/2010 00:35'!
clearTable
^self costume renderedMorph clearTable! !

!SpreadsheetPlayer methodsFor: 'file i/o' stamp: 'Richo 5/4/2011 15:31'!
exportDataToFile
[self costume renderedMorph saveCSVFile: fileName separator: separator] on: Error do: [:err| ^self inform: err description]! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 11:07'!
getColumnIndex
columnIndex > self getTotalColumns ifTrue: [columnIndex := self getTotalColumns].
^ columnIndex! !

!SpreadsheetPlayer methodsFor: 'file i/o' stamp: 'Richo 6/17/2010 22:34'!
getCsvFileName
	^fileName! !

!SpreadsheetPlayer methodsFor: 'file i/o' stamp: 'Richo 6/18/2010 11:54'!
getCsvSeparator
^separator asString! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 11:24'!
getCurrentCharacters
^self costume renderedMorph cellValueAt: self getRowIndex @ self getColumnIndex! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 11:24'!
getCurrentNumericValue
^self costume renderedMorph cellNumericValueAt: self getRowIndex @ self getColumnIndex! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 11:07'!
getRowIndex
rowIndex > self getTotalRows ifTrue: [rowIndex := self getTotalRows].
^ rowIndex! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 10:59'!
getTotalColumns
^self costume renderedMorph numberOfColumns! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 10:59'!
getTotalRows
^self costume renderedMorph numberOfRows! !

!SpreadsheetPlayer methodsFor: 'file i/o' stamp: 'Richo 6/29/2010 11:27'!
getWriteMode
^self costume renderedMorph overwrite ifTrue: [#overwrite] ifFalse: [#append]! !

!SpreadsheetPlayer methodsFor: 'file i/o' stamp: 'Richo 7/26/2010 17:13'!
importDataFromFile
[[self costume renderedMorph openCSVFile: fileName separator: separator] 
on: FileDoesNotExistException do: [:err | self inform: 'The file doesn''t exist.']]
on: Error do: [:err | self inform: 'An error occurred. Please check if the file''s format is CSV.'].! !

!SpreadsheetPlayer methodsFor: 'initialize' stamp: 'Richo 4/12/2011 13:36'!
initialize
super initialize.
rowIndex := 1.
columnIndex := 1.
separator := $;.
fileName := 'fileName.csv'! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 11:08'!
setColumnIndex: aNumber 
(aNumber > self getTotalColumns)
	ifTrue: [columnIndex := 1.^self].
aNumber < 1
	ifTrue: [columnIndex := self getTotalColumns.^self].
columnIndex := aNumber.! !

!SpreadsheetPlayer methodsFor: 'file i/o' stamp: 'Richo 6/17/2010 22:36'!
setCsvFileName: aString
	fileName := aString! !

!SpreadsheetPlayer methodsFor: 'file i/o' stamp: 'Richo 4/12/2011 13:35'!
setCsvSeparator: aString
separator := aString at: 1 ifAbsent: [$;]! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 11:25'!
setCurrentCharacters: aString
^self costume renderedMorph cellValueAt: self getRowIndex @ self getColumnIndex put: aString! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 11:25'!
setCurrentNumericValue: aNumber
^self costume renderedMorph cellNumericValueAt: self getRowIndex @ self getColumnIndex put: aNumber! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 11:08'!
setRowIndex: aNumber 
(aNumber > self getTotalRows)
	ifTrue: [rowIndex := 1.^self].
aNumber < 1
	ifTrue: [rowIndex := self getTotalRows.^self].
rowIndex := aNumber.! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 10:59'!
setTotalColumns: aNumber
^self costume renderedMorph numberOfColumns: (aNumber max: 1)! !

!SpreadsheetPlayer methodsFor: 'spreadsheet' stamp: 'Richo 6/18/2010 10:59'!
setTotalRows: aNumber
^self costume renderedMorph numberOfRows: (aNumber max: 1)! !

!SpreadsheetPlayer methodsFor: 'file i/o' stamp: 'Richo 6/29/2010 11:28'!
setWriteMode: aSymbol
| morph |
morph := self costume renderedMorph.
aSymbol caseOf: {
	[#overwrite] -> [morph setOverwrite].
	[#append] -> [morph setAppend]
} otherwise: [].! !

TilePadMorph subclass: #EtoysTileSource
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-Graphing'!

!EtoysTileSource methodsFor: 'initializing' stamp: 'Richo 9/2/2011 18:01'!
initialize
	super initialize.
	self setType: #Number;
		addMorphBack: ((Vocabulary vocabularyNamed: 'Number') defaultArgumentTile height: 32);
		hResizing: #shrinkWrap; height: 32; color: Color transparent;
		borderWidth: 0; layoutInset: (0@0); cellInset: 0@0; color: (Color r: 0.806 g: 1.0 b: 0.806)! !

!EtoysTileSource methodsFor: 'evaluating' stamp: 'Richo 9/5/2011 16:38'!
value
	| tile |
	^ [tile := submorphs first.
		Compiler
			evaluate: tile codeString
			for: (tile associatedPlayer ifNil: [self assuredPlayer])
			logged: false]
		on: Error
		do: [0]! !

Morph subclass: #BasicSpreadsheetMorph
	instanceVariableNames: 'numberOfRows numberOfColumns overwrite'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-Spreadsheet'!

!BasicSpreadsheetMorph commentStamp: 'Richo 9/5/2011 16:37' prior: 0!
A basic spreadsheet.!

!BasicSpreadsheetMorph class methodsFor: 'viewer categories' stamp: 'Richo 4/12/2011 13:34'!
additionsToViewerCategories
^ #(
(#spreadsheet (
#(command clearTable 'Deletes the contents of all cells')
#(command clearRow 'Deletes the contents of the current row cells' )
#(command clearColumn 'Deletes the contents of the current columns cells' )
#(slot totalRows 'Lets you change the amount of rows in the spreadsheet' Number readWrite Player getTotalRows Player setTotalRows:  )
#(slot totalColumns 'Lets you change the amount of columns in the spreadsheet' Number readWrite Player getTotalColumns Player setTotalColumns:  )
#(slot rowIndex 'Sets the current row' Number readWrite Player getRowIndex Player setRowIndex:  )
#(slot columnIndex 'Sets the current column' Number readWrite Player getColumnIndex Player setColumnIndex:  )
#(slot currentCharacters 'Lets you read and modify the characters of the current cell' String readWrite Player getCurrentCharacters Player setCurrentCharacters:  )
#(slot currentNumericValue 'Lets you read and modify the numeric value of the current cell' Number readWrite Player getCurrentNumericValue Player setCurrentNumericValue:  )
))

(#'data i/o' (
#(slot writeMode 'Lets you choose a write mode (overwrite or append)' WriteMode readWrite Player getWriteMode Player setWriteMode:  )
#(slot csvSeparator 'Lets you choose a character to act as a separator between cells' String readWrite Player getCsvSeparator Player setCsvSeparator:  )
#(slot csvFileName 'Sets the name of the file to import/export data' String readWrite Player getCsvFileName Player setCsvFileName:  )
#(command importDataFromFile 'Reads the data from the file and loads it in the spreadsheet')
#(command exportDataToFile 'Writes the data from the spreadsheet into the file')
))

)! !

!BasicSpreadsheetMorph class methodsFor: 'class initialization' stamp: 'Richo 4/12/2011 13:32'!
initialize
"
self initialize.
"
Vocabulary addStandardVocabulary: (SymbolListType new vocabularyName: #WriteMode;
			 symbols: #(#overwrite #append)).! !

!BasicSpreadsheetMorph class methodsFor: 'instance creation' stamp: 'Richo 9/5/2011 15:48'!
new
	^ self rows: 10 columns: 5! !

!BasicSpreadsheetMorph class methodsFor: 'instance creation' stamp: 'Richo 9/5/2011 15:49'!
rows: numberOfRows columns: numberOfColumns
	^self basicNew initializeWithRows: numberOfRows columns: numberOfColumns! !

!BasicSpreadsheetMorph methodsFor: 'adding/removing cells' stamp: 'Richo 6/17/2010 18:33'!
addColumn
	numberOfColumns := numberOfColumns + 1.
	self submorphsDo: [:row | row addCell]! !

!BasicSpreadsheetMorph methodsFor: 'adding/removing cells' stamp: 'Richo 6/17/2010 20:48'!
addColumns: aNumber
	aNumber timesRepeat: [self addColumn]! !

!BasicSpreadsheetMorph methodsFor: 'adding/removing cells' stamp: 'Richo 6/18/2010 10:42'!
addRow
	| newRow |
	numberOfRows := numberOfRows + 1.
	newRow := self newRow.
	1 to: numberOfColumns do: [:i | 
		newRow column: i width: (self widthOfCell: i)	
	].
	self addMorphBack: newRow! !

!BasicSpreadsheetMorph methodsFor: 'adding/removing cells' stamp: 'Richo 6/17/2010 20:48'!
addRows: aNumber
	aNumber timesRepeat: [self addRow]! !

!BasicSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/18/2010 11:14'!
cellNumericValueAt: aPoint
^(submorphs at: aPoint x) cellNumericValueAt: aPoint y! !

!BasicSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/18/2010 11:14'!
cellNumericValueAt: aPoint put: aNumber
 (submorphs at: aPoint x) cellNumericValueAt: aPoint y put: aNumber! !

!BasicSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/18/2010 11:11'!
cellValueAt: aPoint
^(submorphs at: aPoint x) cellValueAt: aPoint y! !

!BasicSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/18/2010 11:11'!
cellValueAt: aPoint put: aString
 (submorphs at: aPoint x) cellValueAt: aPoint y put: aString! !

!BasicSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 4/12/2011 15:03'!
clearColumn: column
	(column > numberOfColumns or: [column <= 0]) ifTrue: [^self error: 'Invalid column' translated].
	1 to: numberOfRows do: [:r| self cellValueAt: r@column put: '']

! !

!BasicSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 4/12/2011 15:03'!
clearRow: row
	(row > numberOfRows or: [row <= 0]) ifTrue: [^self error: 'Invalid row' translated].
	1 to: numberOfColumns do: [:c| self cellValueAt: row@c put: '']

! !

!BasicSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/28/2010 23:02'!
clearTable
1 to: numberOfRows do: [:r|
	1 to: numberOfColumns do: [:c|
self cellValueAt: r@c put: ''

	]
]
! !

!BasicSpreadsheetMorph methodsFor: 'cell extent change' stamp: 'Richo 6/17/2010 23:45'!
column: index width: aNumber
self submorphsDo: [:each| each column: index width: aNumber]! !

!BasicSpreadsheetMorph methodsFor: 'halo' stamp: 'Richo 6/17/2010 23:48'!
defersHaloToInteriorMorph: aMorph
^true! !

!BasicSpreadsheetMorph methodsFor: 'initialize' stamp: 'Richo 6/17/2010 19:21'!
initializeSubmorphs
numberOfRows timesRepeat: [self addMorph: self newRow]! !

!BasicSpreadsheetMorph methodsFor: 'initialize' stamp: 'Richo 9/5/2011 15:49'!
initializeWithRows: nRows columns: nColumns
	numberOfRows := nRows.
	numberOfColumns := nColumns.
	self initialize.
	self layoutPolicy: TableLayout new; listDirection: #topToBottom.
	self hResizing: #shrinkWrap;
		vResizing: #shrinkWrap.
	self color: Color black.
	self initializeSubmorphs; setOverwrite
! !

!BasicSpreadsheetMorph methodsFor: 'initialize' stamp: 'Richo 6/17/2010 20:53'!
newPlayerInstance
	^SpreadsheetPlayer newUserInstance! !

!BasicSpreadsheetMorph methodsFor: 'initialize' stamp: 'Richo 6/17/2010 19:21'!
newRow
	^(SpreadsheetRowMorph numberOfCells: numberOfColumns)! !

!BasicSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/17/2010 20:46'!
numberOfColumns
^numberOfColumns ! !

!BasicSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 4/12/2011 15:03'!
numberOfColumns: aNumber
| diff |
aNumber <= 0 ifTrue: [^self error: 'Spreadsheet must contain at least one column' translated].
diff := aNumber - numberOfColumns.
diff > 0 ifTrue: [self addColumns: diff] ifFalse: [self removeColumns: diff negated]
! !

!BasicSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/17/2010 20:46'!
numberOfRows
^numberOfRows ! !

!BasicSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 4/12/2011 15:03'!
numberOfRows: aNumber
| diff |
aNumber <= 0 ifTrue: [^self error: 'Spreadsheet must contain at least one row' translated].
diff := aNumber - numberOfRows.
diff > 0 ifTrue: [self addRows: diff] ifFalse: [self removeRows: diff negated]
! !

!BasicSpreadsheetMorph methodsFor: 'i/o' stamp: 'Richo 4/4/2012 15:41'!
openCSVFile: filePath separator: separator
	FileStream 
		readOnlyFileNamed: filePath 
		do: [:file || values rows columns |
			values := CSVParser parse: file delimiter: separator.
			rows := values size.
			columns := values first size.
			self numberOfRows: rows;
				numberOfColumns: columns.
			1 to: rows do: [:r|
				1 to: columns do: [:c |
					self cellValueAt: r@c put: ((values at: r) at: c ifAbsent: '')]]].! !

!BasicSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/29/2010 11:29'!
overwrite
"Boolean. true if you want to overwrite the file, false if you want to append"
^overwrite! !

!BasicSpreadsheetMorph methodsFor: 'adding/removing cells' stamp: 'Richo 6/17/2010 18:35'!
removeColumn
	numberOfColumns := numberOfColumns - 1.
	self submorphsDo: [:row | row removeCell]! !

!BasicSpreadsheetMorph methodsFor: 'adding/removing cells' stamp: 'Richo 6/17/2010 20:49'!
removeColumns: aNumber
	aNumber timesRepeat: [self removeColumn]! !

!BasicSpreadsheetMorph methodsFor: 'adding/removing cells' stamp: 'Richo 6/17/2010 18:38'!
removeRow
	numberOfRows := numberOfRows - 1.
	self removeMorph: self submorphs last! !

!BasicSpreadsheetMorph methodsFor: 'adding/removing cells' stamp: 'Richo 6/17/2010 20:48'!
removeRows: aNumber
	aNumber timesRepeat: [self removeRow]! !

!BasicSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 9/8/2011 13:37'!
rows
	| rows |
	rows := (1 to: numberOfRows)
				collect: [:r | (1 to: numberOfColumns)
						collect: [:c | self cellValueAt: r @ c]].
	"Filter empty cells in the bottom"
	[rows notEmpty
		and: [(rows last
				select: [:each | each notEmpty]) isEmpty]]
		whileTrue: [rows := rows allButLast].
	^ rows! !

!BasicSpreadsheetMorph methodsFor: 'i/o' stamp: 'Richo 9/8/2011 13:42'!
saveCSVFile: filePath separator: separator
	FileStream 
		perform: (self overwrite ifTrue: [#forceNewFileNamed:do:] ifFalse: [#fileNamed:do:])
		with: filePath
		with: [:file |
			file setToEnd.
			self rows do: [:row |
				row do: [:cell | file 
						nextPut: $";
						nextPutAll: (cell copyReplaceAll: '"' with: '""');
						nextPut: $"] 
					separatedBy: [file nextPut: separator].
				file cr]]! !

!BasicSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/29/2010 11:29'!
setAppend
overwrite := false! !

!BasicSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/29/2010 11:29'!
setOverwrite
overwrite := true! !

!BasicSpreadsheetMorph methodsFor: 'cell extent change' stamp: 'Richo 6/18/2010 10:42'!
widthOfCell: index
	^((submorphs first) submorphs at: index) width! !

Morph subclass: #DirectoryEntryMorph
	instanceVariableNames: 'entry'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-FileList'!

DirectoryEntryMorph class
	instanceVariableNames: 'directoryIconForm fileIconForm'!

DirectoryEntryMorph class
	instanceVariableNames: 'directoryIconForm fileIconForm'!

!DirectoryEntryMorph class methodsFor: 'as yet unclassified' stamp: 'Richo 7/22/2010 16:06'!
containingDirectory
^self basicNew initialize! !

!DirectoryEntryMorph class methodsFor: 'forms' stamp: 'Richo 7/22/2010 14:00'!
directoryIconForm
"'Gartoon icons' by Zeus Box (Kuswanto)
Homepage:	http://www.zeusbox.org/"
^directoryIconForm ifNil: [directoryIconForm :=  (Form
	extent: 16@16
	depth: 32
	fromArray: #( 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 2941860392 3210295848 3211941427 3211941427 1884895784 542718504 0 0 0 0 0 0 0 0 0 0 4285946421 4289040202 4285288241 4289040202 4290290771 4287789378 3479521069 1884895784 542718504 0 0 0 0 0 0 0 4017706795 4294820390 4293773898 4289040202 4285288241 4291013197 4294042476 4290948695 4287196989 3478731304 1884895784 811153960 0 0 0 0 3210295848 4294885675 4294950692 4294755632 4290290771 4286798632 4294950692 4294691385 4294431563 4293514334 4290290771 4285946421 3747166760 2153331240 274283048 0 2404989480 4292848180 4294951216 4294950957 4293644376 4284695080 4292913199 4294951213 4294950954 4294950694 4294755633 4294431563 4292921690 4287196989 3210295848 0 1348024872 4290745403 4294951740 4294951737 4294627147 4289698126 4284695852 4287456556 4290875184 4294293810 4294951215 4294950955 4294950695 4294302299 4285288241 0 542718504 4288049721 4294952519 4294952260 4294952257 4294627409 4294302046 4290948180 4287789378 4285288241 4286798892 4294294071 4294951477 4294496847 4286538809 1056964617 0 3748614706 4294888020 4294953040 4294952781 4294952778 4294952519 4294952261 4294822216 4294497367 4289040202 4286799150 4294952258 4294497366 4286538809 1291845641 0 2673424936 4292127314 4294953564 4294953561 4294953302 4294953043 4294953040 4294952782 4294952779 4294367327 4286538809 4292914503 4294497628 4286538809 1662849811 0 1348024872 4288050240 4294954343 4294954085 4294953826 4294953823 4294953564 4294953561 4294953302 4294822999 4290290771 4288837950 4294497891 4286538809 2049908249 0 0 3210295848 4290746709 4294889584 4294954605 4294954347 4294954344 4294954085 4294954082 4294953823 4294432870 4286011188 4294628713 4286538809 2049908249 0 0 274283048 2941860392 4285946421 4290551381 4293971053 4294889587 4294954865 4294954606 4294954347 4294889064 4287854657 4292655203 4286538809 2049908249 0 0 0 0 542718504 1884895784 3210295848 4286538809 4289828432 4292590436 4294694514 4294889587 4293182822 4290551384 4286538809 1714889756 0 0 0 0 0 0 0 83886089 1431582502 2572367398 3730719275 4287196989 4289040202 4291541339 4285946421 1529814553 0 0 0 0 0 0 0 0 0 0 83886089 589830679 1564289054 2622041890 2049908249 570425353)
	offset: 0@0)]! !

!DirectoryEntryMorph class methodsFor: 'forms' stamp: 'Richo 7/22/2010 14:00'!
fileIconForm
"'Gartoon icons' by Zeus Box (Kuswanto)
Homepage:	http://www.zeusbox.org/"
^fileIconForm ifNil: [fileIconForm := (Form
	extent: 16@16
	depth: 32
	fromArray: #( 0 0 0 0 2940030567 4284575620 4285430413 3477888115 2403159655 1346195047 272453223 0 0 0 0 0 0 0 0 1077759591 4284575620 4291353331 4289381868 4290696177 4290168283 4288127665 4284575620 3208466023 2151501415 809324135 0 0 0 0 0 2940030567 4289379025 4288330729 4288133608 4288002024 4287804903 4287673575 4288922091 4290103265 4288456889 4284575620 3745336935 540888679 0 0 540888679 4283786362 4291024626 4289053419 4288921835 4288724970 4288527850 4288396265 4288199145 4288067816 4287870695 4290499056 4287798186 2284994645 0 0 1883065959 4288785595 4289973486 4289841902 4289644781 4289447661 4289316332 4289119212 4288987627 4288790506 4288593386 4292141814 4284575620 1578441251 0 0 3745336935 4292403952 4290696177 4290564592 4290367728 4290236143 4290039023 4289841902 4289710574 4289513453 4289381868 4292665834 3728296288 721420288 0 809324135 4286219671 4292273142 4291484659 4291287539 4291090418 4290959090 4290761969 4290630385 4290433264 4290236143 4291287539 4289376957 2603366474 402653184 0 2403159655 4290692559 4292404470 4292207606 4292010485 4292733176 4293258745 4294047228 4293981692 4294967295 4294967295 4294967295 4285430413 1780228911 0 0 3746257778 4294244349 4293127417 4293390330 4294507262 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294178038 3912911715 805306368 0 809324135 4287008928 4294244605 4294507261 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4290955472 2972794707 402653184 0 2403159655 4290166214 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4287008928 2233806654 83886080 0 3208466023 4294178038 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4099236985 973078528 0 0 2940030567 4287798186 4291810265 4294178038 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4292599523 3342222683 570425344 0 0 624247897 2788113487 3543615326 4097592421 4283786362 4286219671 4288587699 4291810265 4294178038 4294967295 4294967295 4287798186 2603366474 234881024 0 0 0 234881024 570425344 889192448 1511398437 2418619461 2972794707 3543615326 4282207847 4282997105 4285430413 4097592421 1511398437 0 0 0 0 0 0 0 0 83886080 318767104 654311424 889192448 1511398437 2048928054 1662261537 570425344 0 0 0)
	offset: 0@0)]! !

!DirectoryEntryMorph class methodsFor: 'instance creation' stamp: 'Richo 7/20/2010 12:21'!
on: aDirectoryEntry 
^(self basicNew setEntry: aDirectoryEntry) initialize! !

!DirectoryEntryMorph methodsFor: 'accessing' stamp: 'Richo 7/22/2010 16:12'!
entryName
	^entry isNil ifTrue: ['..'] ifFalse: [entry name]! !

!DirectoryEntryMorph methodsFor: 'accessing' stamp: 'Richo 7/22/2010 16:08'!
icon
^self isDirectory ifTrue: [self class directoryIconForm] ifFalse: [self class fileIconForm]! !

!DirectoryEntryMorph methodsFor: 'initialization' stamp: 'Richo 7/22/2010 16:02'!
initialize
	super initialize.
	self extent: 1@1.
	self layoutPolicy: TableLayout new;
		 listDirection: #leftToRight.
	self hResizing: #spaceFill;
		 vResizing: #shrinkWrap.
	self color: Color white.
	self initializeSubmorphs! !

!DirectoryEntryMorph methodsFor: 'initialization' stamp: 'Richo 7/22/2010 16:08'!
initializeSubmorphs
self addMorphBack: (SketchMorph withForm: self icon);
	addMorphBack: ' ' asMorph;
	addMorphBack: self entryName asMorph;
	addMorphBack: AlignmentMorph newVariableTransparentSpacer ! !

!DirectoryEntryMorph methodsFor: 'testing' stamp: 'Richo 7/22/2010 16:08'!
isDirectory
^entry isNil or: [entry isDirectory]! !

!DirectoryEntryMorph methodsFor: 'private' stamp: 'Richo 7/20/2010 12:22'!
setEntry: aDirectoryEntry
entry := aDirectoryEntry ! !

Morph subclass: #FileListMorph
	instanceVariableNames: 'directory selectedEntry filter showDirectories'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-FileList'!

!FileListMorph class methodsFor: 'viewer categories' stamp: 'Richo 4/12/2011 15:26'!
additionsToViewerCategories
^ #(
(#'file list' (
#(command openDirectory 'Opens the selected item (assuming it''s a directory)')
#(command updateCurrentDirectory 'Updates the contents of the list')
#(slot entryIndex 'Lets you modify the selected item' Number readWrite Player getEntryIndex Player setEntryIndex:)
#(slot entryName 'Shows the name of the selected item' String readOnly Player getEntryName Player unused)
#(slot entryPath 'Shows the full path of the selected item' String readOnly Player getEntryPath Player unused)
#(slot entryExtension 'Shows the extension of the selected item' String readOnly Player getEntryExtension Player unused)
#(slot entriesCount 'Shows how many items are in the current directory' Number readOnly Player getEntriesCount Player unused)
#(slot isDirectory 'Shows if the selected item is a directory or a file' Boolean readOnly Player getIsDirectory Player unused)
#(slot showDirectories 'Set to true if you want to show directories in the list' Boolean readWrite Player getShowDirectories Player setShowDirectories:)
#(slot filter 'A string you can use to filter the list' String readWrite Player getFilter Player setFilter:)
))
)! !

!FileListMorph class methodsFor: 'instance creation' stamp: 'Richo 7/22/2010 15:10'!
new
^self on: FileDirectory default! !

!FileListMorph class methodsFor: 'instance creation' stamp: 'Richo 7/20/2010 12:11'!
on: aDirectory
^(self basicNew setDirectory: aDirectory) initialize! !

!FileListMorph methodsFor: 'testing' stamp: 'Richo 7/22/2010 20:09'!
currentDirectoryIsRoot
^directory localName = FileDirectory root localName! !

!FileListMorph methodsFor: 'accessing' stamp: 'Richo 7/22/2010 20:08'!
currentDirectoryName
^self currentDirectoryIsRoot
				ifTrue: [directory localName]
				ifFalse: ['..' , directory slash , directory localName]! !

!FileListMorph methodsFor: 'accessing' stamp: 'Richo 7/25/2010 16:45'!
directories
	showDirectories ifFalse: [^#()].
	^ (self currentDirectoryIsRoot ifTrue: [#()] ifFalse: [{nil}]) , (directory entries
			select: [:each | each isDirectory]).
! !

!FileListMorph methodsFor: 'accessing' stamp: 'Richo 7/25/2010 18:22'!
directory: aFileDirectory
	| newSelectedEntry |
	(self isValidDirectory: aFileDirectory)
		ifFalse: [^self].
	directory := aFileDirectory.
	self
		submorphsDo: [:m | m delete].
	self initializeSubmorphs.
	self entries isEmpty ifTrue: [selectedEntry := nil. ^self].
	newSelectedEntry := self entries detect: [:each | each entryName = self selectedEntryName] ifNone: nil.
	self selectedEntry: (newSelectedEntry ifNil: [self entries first])! !

!FileListMorph methodsFor: 'event handling' stamp: 'Richo 7/22/2010 18:29'!
doubleClick: evt fromMorph: aDirectoryEntryMorph 
	aDirectoryEntryMorph isDirectory
		ifFalse: [^ self].
	aDirectoryEntryMorph entryName = '..'
		ifTrue: [^ self directory: directory containingDirectory].
	self
		directory: (directory directoryNamed: aDirectoryEntryMorph entryName)! !

!FileListMorph methodsFor: 'accessing' stamp: 'Richo 7/25/2010 16:35'!
entries
	^ self submorphs allButFirst! !

!FileListMorph methodsFor: 'accessing' stamp: 'Richo 7/22/2010 18:18'!
entriesCount
	^ self entries size! !

!FileListMorph methodsFor: 'accessing' stamp: 'Richo 7/25/2010 16:36'!
files
	^ directory entries
		select: [:each | each isDirectory not and: [filter match: each name]]! !

!FileListMorph methodsFor: 'accessing' stamp: 'Richo 7/25/2010 16:43'!
filter
^filter! !

!FileListMorph methodsFor: 'accessing' stamp: 'Richo 7/25/2010 16:44'!
filter: aString
filter := aString.
self updateCurrentDirectory ! !

!FileListMorph methodsFor: 'initialization' stamp: 'Richo 7/25/2010 16:43'!
initialize
	super initialize.
	self layoutPolicy: TableLayout new;
		 listDirection: #topToBottom.
	self hResizing: #shrinkWrap;
		 vResizing: #shrinkWrap.
	self color: Color transparent.
	filter := '*'.
	showDirectories := true.
	self initializeSubmorphs.
	self selectedEntry: self entries first ! !

!FileListMorph methodsFor: 'initialization' stamp: 'Richo 7/25/2010 16:45'!
initializeSubmorphs
	self addMorphBack: (AlignmentMorph newRow vResizing: #shrinkWrap;
			 color: Color transparent;
			 addMorphBack: (self currentDirectoryName asMorph emphasis: TextEmphasis bold emphasisCode);
			 addMorphBack: AlignmentMorph newVariableTransparentSpacer).
	self directories , self files
		do: [:entry | self addMorphBack: ((DirectoryEntryMorph on: entry)
					on: #mouseUp
					send: #mouseUp:fromMorph:
					to: self;
					
					on: #doubleClick
					send: #doubleClick:fromMorph:
					to: self)]! !

!FileListMorph methodsFor: 'testing' stamp: 'Richo 7/26/2010 23:36'!
isValidDirectory: aFileDirectory 
	^SecurityManager default isInRestrictedMode ifTrue: [
	aFileDirectory pathName = FileDirectory default pathName
		or: [FileDirectory default directoryExists: aFileDirectory pathName]]
ifFalse: [true]! !

!FileListMorph methodsFor: 'event handling' stamp: 'Richo 7/22/2010 15:23'!
mouseUp: evt fromMorph: aDirectoryEntryMorph
	self selectedEntry: aDirectoryEntryMorph! !

!FileListMorph methodsFor: 'initialization' stamp: 'Richo 7/22/2010 16:35'!
newPlayerInstance
^FileListPlayer newUserInstance! !

!FileListMorph methodsFor: 'actions' stamp: 'Richo 4/12/2011 15:15'!
openDirectory
	self selectedEntryIsDirectory
		ifFalse: [^ self inform: 'Selected entry is not a directory' translated].
	self selectedEntryName = '..'
		ifTrue: [^ self directory: directory containingDirectory].
	self
		directory: (directory directoryNamed: self selectedEntryName)! !

!FileListMorph methodsFor: 'event handling' stamp: 'Richo 7/22/2010 15:24'!
selectedEntry: aDirectoryEntryMorph
	selectedEntry ifNotNilDo: [:old | old color: Color white].
	selectedEntry := aDirectoryEntryMorph color: Color blue muchLighter.! !

!FileListMorph methodsFor: 'accessing' stamp: 'Richo 7/23/2010 15:44'!
selectedEntryExtension
	^FileDirectory extensionFor: self selectedEntryName ! !

!FileListMorph methodsFor: 'accessing' stamp: 'Richo 7/22/2010 18:18'!
selectedEntryIndex
	^ self entries indexOf: selectedEntry! !

!FileListMorph methodsFor: 'accessing' stamp: 'Richo 7/22/2010 18:18'!
selectedEntryIndex: aNumber 
	self
		selectedEntry: (self entries at: aNumber)! !

!FileListMorph methodsFor: 'accessing' stamp: 'Richo 7/25/2010 18:21'!
selectedEntryIsDirectory
^selectedEntry notNil and: [selectedEntry isDirectory]! !

!FileListMorph methodsFor: 'accessing' stamp: 'Richo 7/25/2010 18:21'!
selectedEntryName
	^selectedEntry isNil ifTrue: [''] ifFalse: [selectedEntry entryName]! !

!FileListMorph methodsFor: 'accessing' stamp: 'Richo 7/22/2010 21:07'!
selectedEntryPath
^self selectedEntryName = '..' ifTrue: [directory containingDirectory pathName] ifFalse: [directory fullPathFor: self selectedEntryName  ]! !

!FileListMorph methodsFor: 'private' stamp: 'Richo 7/20/2010 12:13'!
setDirectory: aDirectory
directory := aDirectory! !

!FileListMorph methodsFor: 'accessing' stamp: 'Richo 7/25/2010 16:43'!
showDirectories
^showDirectories! !

!FileListMorph methodsFor: 'accessing' stamp: 'Richo 7/25/2010 16:44'!
showDirectories: aBoolean
showDirectories := aBoolean.
self updateCurrentDirectory ! !

!FileListMorph methodsFor: 'stepping' stamp: 'Richo 7/23/2010 16:00'!
step
(self isValidDirectory: directory) ifFalse: [self directory: FileDirectory default].! !

!FileListMorph methodsFor: 'stepping' stamp: 'Richo 7/23/2010 16:00'!
stepTime
^200! !

!FileListMorph methodsFor: 'actions' stamp: 'Richo 7/25/2010 16:42'!
updateCurrentDirectory
	self directory: directory! !

Morph subclass: #PEBasicGraphMorph
	instanceVariableNames: 'form pen cartesianBounds points startTime isGraphing source'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-Graphing'!

!PEBasicGraphMorph class methodsFor: 'as yet unclassified' stamp: 'Richo 9/2/2011 11:26'!
on: aValuableObject
	^ self basicNew initializeWith: aValuableObject! !

!PEBasicGraphMorph methodsFor: 'accessing' stamp: 'Richo 5/11/2011 20:05'!
addPoint: aPoint
	points size = 10000 ifTrue: [points removeFirst].
	points add: aPoint ! !

!PEBasicGraphMorph methodsFor: 'accessing' stamp: 'Richo 5/6/2011 12:56'!
cartesianBounds
	^ cartesianBounds ! !

!PEBasicGraphMorph methodsFor: 'accessing' stamp: 'Richo 5/6/2011 11:41'!
cartesianBounds: aRectangle 
	cartesianBounds := aRectangle! !

!PEBasicGraphMorph methodsFor: 'calculations' stamp: 'Richo 5/11/2011 14:07'!
cartesianPointToPixel: aPoint in: graphBounds
	^ (self cartesianXToPixel: aPoint x in: graphBounds)
		@ (self cartesianYToPixel: aPoint y in: graphBounds)! !

!PEBasicGraphMorph methodsFor: 'calculations' stamp: 'Richo 5/11/2011 14:08'!
cartesianXToPixel: aNumber in: graphBounds
	^ (graphBounds left + (aNumber - cartesianBounds left * graphBounds width / cartesianBounds width)) rounded! !

!PEBasicGraphMorph methodsFor: 'calculations' stamp: 'Richo 5/11/2011 14:08'!
cartesianYToPixel: aNumber in: graphBounds
	^ (graphBounds bottom - (aNumber - cartesianBounds top * graphBounds height / cartesianBounds height)) rounded! !

!PEBasicGraphMorph methodsFor: 'scale' stamp: 'Richo 9/5/2011 15:08'!
changeScale
	| minValue maxValue |
	minValue := UIManager request: 'Min value:' initialAnswer: self minVal asString.
	minValue isEmpty ifTrue: [^ self].
	[self minVal: (Compiler evaluate: minValue)] on: Error do: ["ignore"].
	maxValue := UIManager request: 'Max value:' initialAnswer: self maxVal asString.
	maxValue isEmpty ifTrue: [^ self].
	[self maxVal: (Compiler evaluate: maxValue)] on: Error do: ["ignore"].! !

!PEBasicGraphMorph methodsFor: 'scale' stamp: 'Richo 9/5/2011 15:07'!
changeScale2
	| minValue maxValue |
	minValue := UIManager request: 'Min value:' initialAnswer: self minVal asString.
	minValue isEmpty ifTrue: [^ self].
	[self minVal: minValue asNumber] on: Error do: ["ignore"].
	maxValue := UIManager request: 'Max value:' initialAnswer: self maxVal asString.
	maxValue isEmpty ifTrue: [^ self].
	[self maxVal: maxValue asNumber] on: Error do: ["ignore"].! !

!PEBasicGraphMorph methodsFor: 'drawing' stamp: 'Richo 9/5/2011 15:17'!
drawDataOn: aCanvas in: aRectangle
	| pointsToDraw |
	aCanvas
		frameRectangle: aRectangle color: Color gray muchLighter;
		line: aRectangle topLeft to: aRectangle bottomLeft color: Color gray;
		line: aRectangle bottomLeft - (0@1) to: aRectangle bottomRight - (0@1) color: Color gray.
	pointsToDraw := self pointsToDrawNow.
	pointsToDraw
		withIndexDo: [:point :index |
			index = 1 ifFalse: [| from to |
				from := self cartesianPointToPixel: (pointsToDraw at: index - 1) in: aRectangle.
				to := self cartesianPointToPixel: point in: aRectangle.
				aCanvas line: from to: to color: self color
			]
		].! !

!PEBasicGraphMorph methodsFor: 'drawing' stamp: 'Richo 5/11/2011 19:18'!
drawLabelsOn: aCanvas
	"REMEMBER: It should return the graph bounds"
	| font yMaxLabelWidth yMinLabelWidth xMaxLabelWidth graphBounds allLabelsHeight xMinLabelWidth |
	font := TextStyle defaultFont.
	yMaxLabelWidth := font widthOfString: self yMaxLabel.
	yMinLabelWidth := font widthOfString: self yMinLabel.
	xMaxLabelWidth := font widthOfString: self xMaxLabel.
	xMinLabelWidth := font widthOfString: self xMinLabel.
	allLabelsHeight := font height.
	graphBounds := Rectangle 
		origin: bounds topLeft 
			+ ({yMaxLabelWidth. yMinLabelWidth. xMinLabelWidth / 2} max rounded @ (allLabelsHeight / 2) rounded) 
			+ (5 @ 0)
		corner: bounds bottomRight 
			- ((xMaxLabelWidth / 2) rounded @ allLabelsHeight)
			- (0 @ 5).
	aCanvas 
		drawString: self yMaxLabel
		at: graphBounds topLeft - (yMaxLabelWidth @ (allLabelsHeight / 2) rounded) - (5 @ 0);
		drawString: self yMinLabel
		at: graphBounds bottomLeft - (yMinLabelWidth @ (allLabelsHeight / 2) rounded) - (5 @ 0);
		drawString: self xMinLabel
		at: graphBounds bottomLeft - ((xMinLabelWidth / 2) rounded @ 0) + (0 @ 5);
		drawString: self xMaxLabel
		at: graphBounds bottomRight - ((xMaxLabelWidth / 2) rounded @ 0) + (0 @ 5).
	^ graphBounds! !

!PEBasicGraphMorph methodsFor: 'drawing' stamp: 'Richo 9/5/2011 15:18'!
drawOn: aCanvas 
	| graphBounds |
	graphBounds := self drawLabelsOn: aCanvas.
	aCanvas clipBy: graphBounds during: [:canvas |
		self drawDataOn: canvas in: graphBounds]
! !

!PEBasicGraphMorph methodsFor: 'accessing' stamp: 'Richo 5/6/2011 10:58'!
extent: newExtent
	form := Form extent: newExtent depth: 32.
	pen := Pen newOnForm: form.
	super extent: newExtent
! !

!PEBasicGraphMorph methodsFor: 'initialization' stamp: 'Richo 9/5/2011 18:25'!
initializeWith: aValuableObject
	self initialize.
	self extent: 800@200.
	source := aValuableObject.
	points := OrderedCollection new.
	cartesianBounds := (0 @ 0 corner: 2000 @ 1023).
	startTime := Time millisecondClockValue.
	isGraphing := false.
! !

!PEBasicGraphMorph methodsFor: 'accessing' stamp: 'Richo 5/11/2011 20:09'!
isGraphing
	^ isGraphing ! !

!PEBasicGraphMorph methodsFor: 'accessing' stamp: 'Richo 9/2/2011 11:40'!
maxVal
	^ cartesianBounds corner y! !

!PEBasicGraphMorph methodsFor: 'accessing' stamp: 'Richo 9/2/2011 11:43'!
maxVal: aNumber
	cartesianBounds := (cartesianBounds origin corner: cartesianBounds corner x @ aNumber).
	self changed! !

!PEBasicGraphMorph methodsFor: 'accessing' stamp: 'Richo 9/2/2011 11:41'!
minVal
	^ cartesianBounds origin y! !

!PEBasicGraphMorph methodsFor: 'accessing' stamp: 'Richo 9/2/2011 11:43'!
minVal: aNumber
	cartesianBounds := (cartesianBounds origin x @ aNumber corner: cartesianBounds corner).
	self changed! !

!PEBasicGraphMorph methodsFor: 'stepping' stamp: 'Richo 5/6/2011 11:57'!
move: aPoint 
	cartesianBounds := cartesianBounds translateBy: aPoint.
! !

!PEBasicGraphMorph methodsFor: 'drawing' stamp: 'Richo 9/2/2011 12:19'!
pointsToDrawNow
	"The logic is a little complicated because this method will be evaluated for every drawing so it *must* be fast and I didn't wanted to iterate the points collection more than once. The first time I found a point between the left and right bounds I also add the previous and the last time I found one I also add the next, so that the graph drawing always reaches the bounds"
	| pointsToDraw firstFound lastFound |
	pointsToDraw := OrderedCollection new.
	firstFound := lastFound := false.
	1 to: points size do: [:index || point |
		point := points at: index.
		(point x >= cartesianBounds left and: [point x <= cartesianBounds right])
			ifTrue: [
				firstFound ifFalse: [
					(points at: index - 1 ifAbsent: nil)
					ifNotNil: [:p | firstFound := true.
						pointsToDraw addLast: p]].
				pointsToDraw addLast: point]
			ifFalse: [
				(firstFound and: [lastFound not])
					ifTrue: [lastFound := true.
						pointsToDraw addLast: point]]].
	^ pointsToDraw! !

!PEBasicGraphMorph methodsFor: 'scrolling' stamp: 'Richo 9/5/2011 15:03'!
scrollTo: n
	| intervals selectedInterval |
	(isGraphing or: [points isEmpty]) ifTrue: [^ self].
	intervals := OrderedCollection new.
	points first x to: points last x - 2000 by: 100 do: [:each |
		intervals add: {each. each + 2000}].
	selectedInterval := intervals at: intervals size - 1 * n + 1 ifAbsent: [^ self].
	cartesianBounds := (selectedInterval first  @ self minVal corner: selectedInterval second @ self maxVal).
	self changed! !

!PEBasicGraphMorph methodsFor: 'stepping' stamp: 'Richo 9/2/2011 10:47'!
startGraphing
	isGraphing := true.
	points := OrderedCollection new.
	cartesianBounds := (0 @ self minVal corner: 2000 @ self maxVal).
	startTime := Time millisecondClockValue.	
	self startStepping! !

!PEBasicGraphMorph methodsFor: 'stepping' stamp: 'Richo 9/2/2011 11:27'!
step
	| now |
	isGraphing ifFalse: [^ self].
	now := Time millisecondClockValue - startTime.
	self cartesianBounds: (self cartesianBounds left @ self minVal corner: self cartesianBounds right @ self maxVal);
		 addPoint: now @ source value.
	now > self cartesianBounds right
		ifTrue: [self move: 100 @ 0].
	self changed! !

!PEBasicGraphMorph methodsFor: 'stepping' stamp: 'Richo 9/2/2011 10:45'!
stepTime
	^ 1! !

!PEBasicGraphMorph methodsFor: 'stepping' stamp: 'Richo 5/11/2011 19:05'!
stopGraphing
	isGraphing := false.
	self stopStepping! !

!PEBasicGraphMorph methodsFor: 'accessing' stamp: 'Richo 5/11/2011 17:10'!
xMaxLabel
	^ (self cartesianBounds right / 1000.0) printShowingDecimalPlaces: 2! !

!PEBasicGraphMorph methodsFor: 'accessing' stamp: 'Richo 5/11/2011 17:10'!
xMinLabel
	^ (self cartesianBounds left / 1000.0) printShowingDecimalPlaces: 2! !

!PEBasicGraphMorph methodsFor: 'accessing' stamp: 'Richo 9/2/2011 10:47'!
yMaxLabel
	^ self maxVal asString! !

!PEBasicGraphMorph methodsFor: 'accessing' stamp: 'Richo 9/2/2011 10:47'!
yMinLabel
	^ self minVal asString! !

Morph subclass: #PEGraphMorph
	instanceVariableNames: 'basicGraph slider'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-Graphing'!

!PEGraphMorph class methodsFor: 'parts bin' stamp: 'Richo 4/10/2012 15:17'!
descriptionForPartsBin
	"Answer a description for use in parts bins"

	^ self partName: 	'Graph' translatedNoop
		categories:		{'Physical Etoys' translatedNoop}
		documentation:	'A graph that draws the input from a variable' translatedNoop! !

!PEGraphMorph class methodsFor: 'instance creation' stamp: 'Richo 9/5/2011 14:20'!
new
	^ self on: EtoysTileSource new! !

!PEGraphMorph class methodsFor: 'instance creation' stamp: 'Richo 9/5/2011 16:12'!
newStandAlone
	^ self new! !

!PEGraphMorph class methodsFor: 'instance creation' stamp: 'Richo 9/5/2011 16:35'!
on: aValuableObject
	^ self basicNew initializeWith: aValuableObject! !

!PEGraphMorph methodsFor: 'initialize-release' stamp: 'Richo 5/11/2011 17:48'!
button: label color: aColor
	^ SimpleButtonMorph new
		label: label;
		borderColor: aColor;
		color: (aColor lighter alpha: 0.5).! !

!PEGraphMorph methodsFor: 'accessing' stamp: 'Richo 5/11/2011 20:32'!
extent: aPoint
	| newExtent |
	newExtent := (aPoint x max: self minimumExtent x) @ (aPoint y max: self minimumExtent y).
	^ super extent: newExtent! !

!PEGraphMorph methodsFor: 'initialize-release' stamp: 'Richo 5/11/2011 19:03'!
inARow: anArray
	^ (AlignmentMorph inARow: anArray) 
		borderWidth: 0;
		extent: 1@1;
		layoutInset: 0;
		cellInset: 5;
		vResizing: #shrinkWrap;
		hResizing: #spaceFill;
		wrapCentering: #center;
		cellPositioning: #center.! !

!PEGraphMorph methodsFor: 'initialize-release' stamp: 'Richo 5/11/2011 20:19'!
initialize
	super initialize.
	self layoutPolicy: TableLayout new;
		rubberBandCells: true;
		listDirection: #topToBottom;
		vResizing: #rigid;
		hResizing: #rigid;
		extent: 800@300;
		layoutInset: 5;
		cellInset: 10;
		borderWidth: 1;
		wrapCentering: #center;
		cellPositioning: #leftCenter;
		color: Color white;
		borderColor: Color gray! !

!PEGraphMorph methodsFor: 'initialize-release' stamp: 'Richo 4/10/2012 16:16'!
initializeWith: valuableObject
	| |
	self initialize.
	basicGraph := (PEBasicGraphMorph on: valuableObject)
						 hResizing: #spaceFill;
						 vResizing: #spaceFill.
	self addMorphBack: (self inARow: {
			'Source: ' translated asMorph emphasis: 1.
			valuableObject asMorph
			});
		addMorphBack: basicGraph;
		addMorphBack: (self inARow: {
			(self button: 'Start' translated color: Color green)
				target: basicGraph; actionSelector: #startGraphing.
			(self button: 'Stop' translated color: Color red)
				target: basicGraph; actionSelector: #stopGraphing.
			(self button: 'Set scale...' translated color: Color blue muchLighter)
				target: basicGraph; actionSelector: #changeScale.
			(slider := self newSliderFrom: 0 to: 1) 
				setScaledValue: 1; 
				hResizing: #spaceFill; 
				actionSelector: #scrollTo:; 
				target: basicGraph
			})! !

!PEGraphMorph methodsFor: 'initialize-release' stamp: 'Richo 5/11/2011 19:25'!
newSliderFrom: minVal to: maxVal 
	^ SimpleSliderMorph new minVal: minVal;
		 maxVal: maxVal;
		 color: Color gray muchLighter;
		 sliderColor: Color blue muchDarker;
		 extent: 200 @ 15! !

!PEGraphMorph methodsFor: 'stepping' stamp: 'Richo 5/11/2011 20:08'!
step
	basicGraph isGraphing ifTrue: [slider setScaledValue: 1]! !

!PEGraphMorph methodsFor: 'stepping' stamp: 'Richo 5/11/2011 20:02'!
stepTime
	^ 200! !

Morph subclass: #PESpreadsheetMorph
	instanceVariableNames: 'time fileName spreadsheet state index source'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-Spreadsheet'!

!PESpreadsheetMorph commentStamp: 'Richo 9/5/2011 16:37' prior: 0!
A spreadsheet that gets its input from a variable, letting you record its value and save it in a file.!

!PESpreadsheetMorph class methodsFor: 'parts bin' stamp: 'Richo 4/10/2012 15:17'!
descriptionForPartsBin
	"Answer a description for use in parts bins"

	^ self partName: 	'Spreadsheet' translatedNoop
		categories:		{'Physical Etoys' translatedNoop}
		documentation:	'A spreadsheet that allows you to read/write csv files' translatedNoop! !

!PESpreadsheetMorph class methodsFor: 'instance creation' stamp: 'Richo 9/5/2011 15:33'!
new
	^ self on: EtoysTileSource new! !

!PESpreadsheetMorph class methodsFor: 'instance creation' stamp: 'Richo 9/5/2011 15:55'!
newStandAlone
	^ self new! !

!PESpreadsheetMorph class methodsFor: 'instance creation' stamp: 'Richo 9/5/2011 15:24'!
on: aValuableObject
	^ self basicNew initializeWith: aValuableObject! !

!PESpreadsheetMorph methodsFor: 'building' stamp: 'Richo 5/6/2011 15:51'!
button: label color: aColor
	^ SimpleButtonMorph new
		label: label;
		borderColor: aColor;
		color: (aColor lighter alpha: 0.5).! !

!PESpreadsheetMorph methodsFor: 'initialize-release' stamp: 'Richo 5/4/2011 15:34'!
delete
	self stopStepping.
	super delete! !

!PESpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 5/4/2011 16:16'!
fileName
	^ fileName contents asString! !

!PESpreadsheetMorph methodsFor: 'building' stamp: 'Richo 5/4/2011 13:31'!
inAColumn: anArray
	^ (AlignmentMorph inAColumn: anArray)
		borderWidth: 0;
		extent: 1@1;
		layoutInset: 0;
		cellInset: 0;
		vResizing: #shrinkWrap;
		hResizing: #spaceFill;
		wrapCentering: #center;
		cellPositioning: #leftCenter.! !

!PESpreadsheetMorph methodsFor: 'building' stamp: 'Richo 5/4/2011 13:31'!
inARow: anArray
	^ (AlignmentMorph inARow: anArray) 
		borderWidth: 0;
		extent: 1@1;
		layoutInset: 0;
		cellInset: 0;
		vResizing: #shrinkWrap;
		hResizing: #spaceFill;
		wrapCentering: #center;
		cellPositioning: #center.! !

!PESpreadsheetMorph methodsFor: 'initialize-release' stamp: 'Richo 9/5/2011 15:50'!
initializeSubmorphs
	self addMorphBack: (
		self inAColumn: {
			self inARow: {
				'Source: ' translated asMorph emphasis: 1.
				self spacer.
			}.
			source asMorph.			
		}).
	self addMorphBack: ((self inAColumn: {
			self inARow: {
				self spacer.
				self text: 'time' translated.
				self text: 'value' translated.
				self spacer.
				}.
			spreadsheet := BasicSpreadsheetMorph rows: self maxRows columns: 2})
			cellPositioning: #center).
	self addMorphBack: (self inAColumn: {
			'Nombre del archivo: ' asMorph.
			fileName := (self text: 'datos.csv')
				wrapFlag: false;
				borderWidth: 1;
				backgroundColor: (Color white alpha: 0.8);
				hResizing: #spaceFill.
			}).
	self addMorphBack: ((self inARow: {
			(self button: 'Iniciar' color: Color green)
				target: self; actionSelector: #startStepping.
			(self button: 'Pausar' color: Color yellow)
				target: self; actionSelector: #pauseStepping.
			(self button: 'Detener' color: Color red)
				target: self; actionSelector: #stopStepping.
			})
			cellInset: 5;
			cellPositioning: #leftCenter)
! !

!PESpreadsheetMorph methodsFor: 'initialize-release' stamp: 'Richo 9/5/2011 15:27'!
initializeWith: aValuableObject
	source := aValuableObject.
	self initialize
		layoutPolicy: TableLayout new;
		rubberBandCells: true;
		listDirection: #topToBottom;
		extent: 1@1;
		vResizing: #shrinkWrap;
		hResizing: #shrinkWrap;
		color: (Color r: 0.645 g: 0.71 b: 1.0);
		cornerStyle: #rounded;
		borderColor: #raised;
		layoutInset: 5;
		cellInset: 10.
	time := 0.
	state := #stopped.
	self initializeSubmorphs.! !

!PESpreadsheetMorph methodsFor: 'stepping' stamp: 'Richo 5/4/2011 15:50'!
isPaused
	^ state == #paused! !

!PESpreadsheetMorph methodsFor: 'stepping' stamp: 'Richo 5/4/2011 15:15'!
isRunning
	^ state == #running! !

!PESpreadsheetMorph methodsFor: 'stepping' stamp: 'Richo 5/4/2011 15:15'!
isStopped
	^ state == #stopped! !

!PESpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 4/10/2012 15:20'!
maxRows
	^ 15! !

!PESpreadsheetMorph methodsFor: 'stepping' stamp: 'Richo 5/4/2011 15:16'!
pauseStepping
	state := #paused.
	super stopStepping! !

!PESpreadsheetMorph methodsFor: 'printing' stamp: 'Richo 5/4/2011 15:16'!
print: aNumber
	^ (aNumber printShowingDecimalPlaces: 2) copyReplaceAll: '.' with: ','! !

!PESpreadsheetMorph methodsFor: 'building' stamp: 'Richo 9/5/2011 15:38'!
spacer
	^ AlignmentMorph newVariableTransparentSpacer! !

!PESpreadsheetMorph methodsFor: 'stepping' stamp: 'Richo 9/5/2011 15:26'!
startStepping
	self isStopped
		ifTrue: [
		time := Time millisecondClockValue.
		[(FileStream forceNewFileNamed: self fileName do: [:file |
			file nextPutAll: 'time' translated;
				nextPut: $;;
				nextPutAll: 'value' translated;
				cr])] on: Error do: [].
		index := 1.
		spreadsheet clearTable; setAppend].
	state := #running.
	super startStepping! !

!PESpreadsheetMorph methodsFor: 'stepping' stamp: 'Richo 9/5/2011 15:24'!
step
	| |
	self isRunning ifFalse: [^ self].
	spreadsheet 
		cellValueAt: index @ 1 put: (self print: (Time millisecondClockValue - time) / 1000);
		cellValueAt: index @ 2 put: (self print: source value).
	index = self maxRows
		ifTrue: [
			spreadsheet 
				saveCSVFile: self fileName separator: $;;
				clearTable
			].
	index := (index \\ self maxRows) + 1! !

!PESpreadsheetMorph methodsFor: 'stepping' stamp: 'Richo 5/4/2011 15:56'!
stepTime
	^ 1! !

!PESpreadsheetMorph methodsFor: 'stepping' stamp: 'Richo 9/5/2011 15:54'!
stopStepping
	self isStopped ifTrue: [^ self].
	state := #stopped.
	spreadsheet
		saveCSVFile: self fileName
		separator: $;.
	super stopStepping! !

!PESpreadsheetMorph methodsFor: 'building' stamp: 'Richo 5/4/2011 13:31'!
text: aString
	^ UserText new
		contents: aString;
		centered;
		autoFit: false;
		setExtentFromHalo: 100@23;
		layoutChanged! !

Morph subclass: #ScrollableFileListMorph
	instanceVariableNames: 'fileList scrollPane'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-FileList'!

!ScrollableFileListMorph class methodsFor: 'viewer categories' stamp: 'Richo 7/22/2010 16:34'!
additionsToViewerCategories
^ (Smalltalk at: #FileListMorph ifAbsent: [^#()]) additionsToViewerCategories ! !

!ScrollableFileListMorph class methodsFor: 'parts bin' stamp: 'Richo 4/12/2011 15:16'!
descriptionForPartsBin
	"Answer a description for use in parts bins"

	^ self partName: 	'File list' translatedNoop
		categories:		{'Tools' translatedNoop}
		documentation:	'A tool you can use to explore your file system' translatedNoop! !

!ScrollableFileListMorph methodsFor: 'dnu' stamp: 'Richo 7/25/2010 16:41'!
doesNotUnderstand: aMessage
	| methodName |
	(fileList respondsTo: aMessage selector)
		ifTrue: [
methodName := aMessage createStubMethod allButLast: 24.
self class compileSilently: methodName , '^ fileList ', methodName classified: #autogenerated.
		^aMessage sentTo: self]
		ifFalse: [super doesNotUnderstand: aMessage].! !

!ScrollableFileListMorph methodsFor: 'accessing' stamp: 'Richo 7/22/2010 17:07'!
entriesCount
^fileList entriesCount! !

!ScrollableFileListMorph methodsFor: 'initialization' stamp: 'Richo 7/22/2010 15:25'!
extent: aPoint
| newExtent |
newExtent := (aPoint x max: 126) @ (aPoint y max: 102).
scrollPane extent: newExtent.
fileList extent: newExtent.
super extent: newExtent
! !

!ScrollableFileListMorph methodsFor: 'autogenerated' stamp: ''!
filter
	^ fileList filter! !

!ScrollableFileListMorph methodsFor: 'autogenerated' stamp: ''!
filter: t1 
	^ fileList filter: t1! !

!ScrollableFileListMorph methodsFor: 'initialization' stamp: 'Richo 7/22/2010 16:26'!
initialize
super initialize.
scrollPane := ScrollPane new extent: 343@508; borderWidth: 2; color: Color white.
scrollPane scroller addMorph: (fileList := FileListMorph new position: 2@5).
scrollPane alwaysShowScrollBars: true.
self extent: scrollPane extent; color: Color transparent.
self addMorph: scrollPane.
self updateScrollPane.
! !

!ScrollableFileListMorph methodsFor: 'initialization' stamp: 'Richo 7/22/2010 16:36'!
newPlayerInstance
	^fileList newPlayerInstance! !

!ScrollableFileListMorph methodsFor: 'actions' stamp: 'Richo 7/22/2010 17:19'!
openDirectory
fileList openDirectory! !

!ScrollableFileListMorph methodsFor: 'accessing' stamp: 'Richo 7/22/2010 21:01'!
selectedEntryExtension
^fileList selectedEntryExtension! !

!ScrollableFileListMorph methodsFor: 'accessing' stamp: 'Richo 7/22/2010 16:57'!
selectedEntryIndex
^fileList selectedEntryIndex ! !

!ScrollableFileListMorph methodsFor: 'accessing' stamp: 'Richo 7/22/2010 17:16'!
selectedEntryIndex: aNumber
fileList selectedEntryIndex: aNumber! !

!ScrollableFileListMorph methodsFor: 'accessing' stamp: 'Richo 7/22/2010 17:01'!
selectedEntryIsDirectory
^fileList selectedEntryIsDirectory! !

!ScrollableFileListMorph methodsFor: 'accessing' stamp: 'Richo 7/22/2010 16:38'!
selectedEntryName
	^fileList selectedEntryName! !

!ScrollableFileListMorph methodsFor: 'accessing' stamp: 'Richo 7/22/2010 21:04'!
selectedEntryPath
^fileList selectedEntryPath
! !

!ScrollableFileListMorph methodsFor: 'autogenerated' stamp: ''!
showDirectories
	^ fileList showDirectories! !

!ScrollableFileListMorph methodsFor: 'autogenerated' stamp: ''!
showDirectories: t1 
	^ fileList showDirectories: t1! !

!ScrollableFileListMorph methodsFor: 'stepping' stamp: 'Richo 7/22/2010 16:27'!
step
self updateScrollPane! !

!ScrollableFileListMorph methodsFor: 'stepping' stamp: 'Richo 7/22/2010 16:27'!
stepTime
^200! !

!ScrollableFileListMorph methodsFor: 'autogenerated' stamp: ''!
updateCurrentDirectory
	^ fileList updateCurrentDirectory! !

!ScrollableFileListMorph methodsFor: 'initialization' stamp: 'Richo 7/22/2010 15:14'!
updateScrollPane
	scrollPane vSetScrollDelta; hSetScrollDelta! !

Morph subclass: #ScrollableSpreadsheetMorph
	instanceVariableNames: 'spreadsheet scrollPane'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-Spreadsheet'!

!ScrollableSpreadsheetMorph commentStamp: 'Richo 9/5/2011 16:37' prior: 0!
A spreadsheet with scrollbars!

!ScrollableSpreadsheetMorph class methodsFor: 'viewer categories' stamp: 'Richo 9/8/2011 13:24'!
additionsToViewerCategories
^ (Smalltalk at: #BasicSpreadsheetMorph ifAbsent: [^#()]) additionsToViewerCategories ! !

!ScrollableSpreadsheetMorph class methodsFor: 'parts bin' stamp: 'Richo 9/5/2011 15:53'!
descriptionForPartsBin
	"Answer a description for use in parts bins"

	^ self partName: 	'Basic spreadsheet' translatedNoop
		categories:		{'Tools' translatedNoop}
		documentation:	'A spreadsheet that allows you to read/write csv files' translatedNoop! !

!ScrollableSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/18/2010 11:17'!
cellNumericValueAt: aPoint 
	^spreadsheet cellNumericValueAt: aPoint! !

!ScrollableSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/18/2010 11:19'!
cellNumericValueAt: aPoint put: aByteString 
	spreadsheet cellNumericValueAt: aPoint put: aByteString ! !

!ScrollableSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/18/2010 11:17'!
cellValueAt: aPoint 
	^spreadsheet cellValueAt: aPoint! !

!ScrollableSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/18/2010 11:18'!
cellValueAt: aPoint put: aByteString 
	spreadsheet cellValueAt: aPoint put: aByteString ! !

!ScrollableSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/29/2010 00:34'!
clearColumn: column
spreadsheet clearColumn: column
! !

!ScrollableSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/29/2010 00:34'!
clearRow: row
spreadsheet clearRow: row! !

!ScrollableSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/29/2010 00:34'!
clearTable
spreadsheet clearTable! !

!ScrollableSpreadsheetMorph methodsFor: 'initialize' stamp: 'Richo 6/17/2010 21:44'!
extent: aPoint
| newExtent |
newExtent := (aPoint x max: 126) @ (aPoint y max: 102).
scrollPane extent: newExtent.
super extent: newExtent
! !

!ScrollableSpreadsheetMorph methodsFor: 'halo' stamp: 'Richo 6/18/2010 10:19'!
handlerForBlueButtonDown: anEvent
"It defers the halo to interior only if the click is inside the bounds of the spreadsheet"
^(spreadsheet boundsInWorld containsPoint: anEvent cursorPoint)
	ifTrue: [nil] ifFalse: [self]
! !

!ScrollableSpreadsheetMorph methodsFor: 'initialize' stamp: 'Richo 9/5/2011 15:50'!
initialize
super initialize.
scrollPane := ScrollPane new extent:  620@300; borderWidth: 0; color: (Color r: 0.419 g: 0.419 b: 0.419).
scrollPane scroller addMorph: (spreadsheet := BasicSpreadsheetMorph new position: 2@5).
scrollPane alwaysShowScrollBars: true.
self extent: scrollPane extent; color: Color transparent.
self addMorph: scrollPane.! !

!ScrollableSpreadsheetMorph methodsFor: 'initialize' stamp: 'Richo 6/17/2010 21:18'!
newPlayerInstance
	^spreadsheet newPlayerInstance! !

!ScrollableSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/17/2010 21:19'!
numberOfColumns
^spreadsheet numberOfColumns! !

!ScrollableSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/17/2010 21:20'!
numberOfColumns: aNumber
	spreadsheet numberOfColumns: aNumber.
	self updateScrollPane! !

!ScrollableSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/17/2010 21:17'!
numberOfRows
^spreadsheet numberOfRows! !

!ScrollableSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/17/2010 21:20'!
numberOfRows: aNumber
	spreadsheet numberOfRows: aNumber.
	self updateScrollPane! !

!ScrollableSpreadsheetMorph methodsFor: 'i/o' stamp: 'Richo 6/29/2010 11:36'!
openCSVFile: filePath separator: aCharacter
	spreadsheet openCSVFile: filePath separator: aCharacter.
	self updateScrollPane ! !

!ScrollableSpreadsheetMorph methodsFor: 'as yet unclassified' stamp: 'Richo 6/29/2010 11:27'!
overwrite
	^spreadsheet overwrite! !

!ScrollableSpreadsheetMorph methodsFor: 'i/o' stamp: 'Richo 6/18/2010 14:28'!
saveCSVFile: filePath separator: aCharacter
	spreadsheet saveCSVFile: filePath separator: aCharacter! !

!ScrollableSpreadsheetMorph methodsFor: 'as yet unclassified' stamp: 'Richo 6/29/2010 11:19'!
setAppend
spreadsheet setAppend! !

!ScrollableSpreadsheetMorph methodsFor: 'as yet unclassified' stamp: 'Richo 6/29/2010 11:28'!
setOverwrite
spreadsheet setOverwrite! !

!ScrollableSpreadsheetMorph methodsFor: 'accessing' stamp: 'Richo 6/17/2010 21:49'!
updateScrollPane
	scrollPane vSetScrollDelta; hSetScrollDelta! !

Morph subclass: #SpreadsheetRowMorph
	instanceVariableNames: 'numberOfCells'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-Spreadsheet'!

!SpreadsheetRowMorph class methodsFor: 'as yet unclassified' stamp: 'Richo 6/17/2010 16:19'!
numberOfCells: aNumber
^(self basicNew setNumberOfCells: aNumber) initialize! !

!SpreadsheetRowMorph methodsFor: 'adding/removing cells' stamp: 'Richo 6/18/2010 10:47'!
addCell
	numberOfCells := numberOfCells + 1.
	self addMorphBack: ((self newTextMorphWithContents: '') height: self height)! !

!SpreadsheetRowMorph methodsFor: 'accessing' stamp: 'Richo 6/29/2010 17:37'!
cellNumericValueAt: index
 ^(submorphs at: index) getNumericValue! !

!SpreadsheetRowMorph methodsFor: 'accessing' stamp: 'Richo 6/28/2010 23:07'!
cellNumericValueAt: index put: aNumber
self cellValueAt: index put: aNumber asString! !

!SpreadsheetRowMorph methodsFor: 'accessing' stamp: 'Richo 6/29/2010 11:40'!
cellValueAt: index
^(submorphs at: index) contents asString! !

!SpreadsheetRowMorph methodsFor: 'accessing' stamp: 'Richo 6/28/2010 23:12'!
cellValueAt: index put: aString
(submorphs at: index) contentsWrapped: aString! !

!SpreadsheetRowMorph methodsFor: 'cell extent change' stamp: 'Richo 6/17/2010 23:46'!
column: index width: aNumber
(submorphs at: index) width: aNumber! !

!SpreadsheetRowMorph methodsFor: 'halo' stamp: 'Richo 6/17/2010 23:48'!
defersHaloToInteriorMorph: aMorph
^true! !

!SpreadsheetRowMorph methodsFor: 'initialize' stamp: 'Richo 6/17/2010 19:18'!
initialize
super initialize.
self layoutPolicy: TableLayout new; listDirection: #leftToRight.
self hResizing: #shrinkWrap;
	vResizing: #shrinkWrap.
self color: Color black.
self initializeSubmorphs! !

!SpreadsheetRowMorph methodsFor: 'initialize' stamp: 'Richo 6/17/2010 18:38'!
initializeSubmorphs
	numberOfCells timesRepeat: [self addMorphBack:(self newTextMorphWithContents: '')]! !

!SpreadsheetRowMorph methodsFor: 'initialize' stamp: 'Richo 4/12/2011 16:56'!
newTextMorphWithContents: aString
	^ TextMorph new autoFit: false;
		 extent: 100 @ 23;
		 borderWidth: 1;
		rightFlush;
		 backgroundColor: Color white;
		crPassesFocus: true;
		highlightsOnFocus: true;
		 contentsWrapped: aString;
		when: #extentChange send: #notifyExtentChange: to: self! !

!SpreadsheetRowMorph methodsFor: 'cell extent change' stamp: 'Richo 6/17/2010 23:47'!
notifyExtentChange: aMorph
| index |
index := self submorphIndexOf: aMorph.
self submorphsDo: [:each | each height: aMorph height].
owner column: index width: aMorph width.! !

!SpreadsheetRowMorph methodsFor: 'adding/removing cells' stamp: 'Richo 6/17/2010 18:36'!
removeCell
	numberOfCells := numberOfCells - 1.
	self removeMorph: self submorphs last! !

!SpreadsheetRowMorph methodsFor: 'private' stamp: 'Richo 6/17/2010 16:20'!
setNumberOfCells: aSmallInteger 
	numberOfCells := aSmallInteger! !

!Project methodsFor: '*physicaletoys' stamp: 'Richo 3/6/2012 16:09'!
acceptDetailsAndStoreProjectAsPE: details
	"The user having hit the ok button in the dialog, send the details back to the project, store the project on the server, "

	self acceptProjectDetails: details.

	self isCurrentProject
		ifTrue:
			["exit, then do the command"
			self
				armsLengthCommand: #storeOnServerAsPEAssumingNameValid
				withDescription: 'Publishing' translated]
		ifFalse:
			[self storeOnServerAsPEWithProgressInfo]! !

!Project methodsFor: '*physicaletoys' stamp: 'Richo 3/6/2012 16:30'!
storeOnServerAsPE
	"Save to disk as an Export Segment.  Then put that file on the server I came from, as a new version.  Version is literal piece of file name.  Mime encoded and http encoded."

	| aMessageSend |
	world setProperty: #optimumExtentFromAuthor toValue: world extent.
	aMessageSend := MessageSend receiver: self selector: #acceptDetailsAndStoreProjectAsPE:.
	(world valueOfProperty: #ProjectDetails)
		ifNotNil:
		[:details | "ensure project info matches real project name"
		details at: 'projectname' put: self name].

	EToyProjectDetailsMorph
		getFullInfoFor: self 
		ifValid: aMessageSend
		expandedFormat: false! !

!Project methodsFor: '*physicaletoys' stamp: 'Richo 3/6/2012 16:10'!
storeOnServerAsPEAssumingNameValid

	"Save to disk as an Export Segment.  Then put that file on the server I came from, as a new version.  Version is literal piece of file name.  Mime encoded and http encoded."
	world setProperty: #optimumExtentFromAuthor toValue: world extent.
	self isCurrentProject ifTrue: ["exit, then do the command"
		Flaps globalFlapTabsIfAny do: [:each | Flaps removeFlapTab: each keepInList: true].
		^ self 
			armsLengthCommand: #storeOnServerAsPEAssumingNameValid
			withDescription: 'Publishing' translated
	].
	self storeOnServerAsPEWithProgressInfo.
! !

!Project methodsFor: '*physicaletoys' stamp: 'Richo 3/7/2012 12:46'!
storeOnServerAsPEInnards
	"Save to disk as an Export Segment.  Then put that file on the server I came from, as a new version.  Version is literal piece of file name.  Mime encoded and http encoded."

	| resp newName primaryServerDirectory serverVersionPair localDirectory localVersionPair myVersionNumber warning maxNumber suppliedPassword oldResourceUrl priorWorld myDepth |
	self assureIntegerVersion.

	"Find out what version"
	primaryServerDirectory _ self primaryServerIfNil: [
		(primaryServerDirectory _ PEFileList modalFolderSelectorForProject: self) ifNil: [^self].
		oldResourceUrl _ self resourceUrl.
		primaryServerDirectory == #localOnly ifTrue: [
			self storeNewPrimaryURL: FileDirectory default url.
			nil
		] ifFalse: [
			self storeNewPrimaryURL: primaryServerDirectory downloadUrl.
			primaryServerDirectory
		].
	].

	localDirectory _ self squeakletDirectory.
	[serverVersionPair _ self class mostRecent: self name onServer: primaryServerDirectory]
		on: LoginFailedException do: [:ex |
			primaryServerDirectory user: nil.^ self inform: 'Login failed.'].
	localVersionPair _ self class mostRecent: self name onServer: localDirectory.
	maxNumber _ myVersionNumber _ self currentVersionNumber.

	ProgressNotification signal: '2:versionsDetected'.

	warning _ ''.
	myVersionNumber < serverVersionPair second ifTrue: [
		warning _ warning,'\There are newer version(s) on the server' translated.
		maxNumber _ maxNumber max: serverVersionPair second.
	].
	myVersionNumber < localVersionPair second ifTrue: [
		warning _ warning,'\There are newer version(s) in the local directory' translated.
		maxNumber _ maxNumber max: localVersionPair second.
	].
	"8 Nov 2000 - only check on the first attempt to publish"
	myVersionNumber = 0 ifTrue: [
		warning isEmpty ifFalse: [
			myVersionNumber = 0 ifTrue: [
				warning _ warning,'\THIS PROJECT HAS NEVER BEEN SAVED' translated.
			].
			warning _ 'WARNING' translated, '\Project: ' translated, self name,warning.
			resp _ (PopUpMenu labels: 'Store anyway\Cancel' translated withCRs) startUpWithCaption: 
				(warning, '\Please cancel, rename this project, and see what is there.' translated) withCRs.
				resp ~= 1 ifTrue: [^ nil]
		].
	].
	version _ self bumpVersion: maxNumber.

	oldResourceUrl
		ifNotNil: [self resourceManager adjustToNewServer: self resourceUrl from: oldResourceUrl].

	"write locally - now zipped automatically"
	Display isVirtualScreen ifTrue: [
		myDepth _ displayDepth.
		displayDepth _ OLPCVirtualScreen preferredScreenDepth.
	].
	newName _ self versionedFileName.
	lastSavedAtSeconds _ Time totalSeconds.
	priorWorld _ ActiveWorld.
	self exportSegmentFileName: newName directory: localDirectory withoutInteraction: false.
	ActiveWorld _ priorWorld.
	(localDirectory readOnlyFileNamed: newName) setFileTypeToObject; close.
	Display isVirtualScreen ifTrue: [
		displayDepth _ myDepth.
	].
	
	ProgressNotification signal: '4:localSaveComplete'.	"3 is deep in export logic"

	primaryServerDirectory ifNotNil: [
		suppliedPassword _ ''.
		Preferences passwordsOnPublish ifTrue: [
			suppliedPassword _ FillInTheBlank requestPassword: 'Project password' translated
		].
		[
		primaryServerDirectory
			writeProject: self
			inFileNamed: newName asFileName
			fromDirectory: localDirectory.
		] on: ProjectPasswordNotification do: [ :ex |
			ex resume: (suppliedPassword ifNil: [''])
		].
	].
	ProgressNotification signal: '9999 save complete'.
	
	PEProject storeProjectNamed: newName in: primaryServerDirectory

	"Later, store with same name on secondary servers.  Still can be race conditions.  All machines will go through the server list in the same order."
	"2 to: servers size do: [:aServer | aServer putFile: local named: newName]."
! !

!Project methodsFor: '*physicaletoys' stamp: 'Richo 3/6/2012 16:10'!
storeOnServerAsPEWithProgressInfo

	"Save to disk as an Export Segment.  Then put that file on the server I came from, as a new version.  Version is literal piece of file name.  Mime encoded and http encoded."
	ProgressInitiationException 
		display: 'Storing Project...'
		during: [:bar |
			[self storeOnServerAsPEInnards]
				on: ProgressNotification
				do: [:e |
					bar value: (e messageText asNumber) / 10.0.
					e resume]]
	! !

AlignmentMorph subclass: #PESupplies
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-Utilities'!

!PESupplies methodsFor: 'initialization' stamp: 'Richo 4/4/2012 15:56'!
initialize
	| tabsPane aBin |
	super initialize.
	self basicInitialize.

	self layoutInset: 0;
		layoutPolicy: ProportionalLayout new;
		hResizing: #shrinkWrap;
		vResizing: #rigid;
		borderWidth: 2; borderColor: Color darkGray;
		color: Color darkGray.

	"Place holder for a tabs or text pane"
	tabsPane := Morph new
		setNameTo: 'TabPane';
		hResizing: #spaceFill;
		yourself.
	tabsPane vResizing: #spaceFill.

	self
		addMorph: tabsPane
		fullFrame: (LayoutFrame
				fractions: (0 @ 0 corner: 1 @ 0)
				offsets: (0 @ 0 corner: 0 @ tabsPane height)).

	aBin := (PartsBin newPartsBinWithOrientation: #leftToRight from: #())
		listDirection: #leftToRight;
		wrapDirection: #topToBottom;
		setNameTo: 'Parts';
		dropEnabled: false;
		vResizing: #spaceFill;
		yourself.

	self
		addMorph: aBin
		fullFrame: (LayoutFrame
				fractions: (0 @ 0 corner: 1 @ 1)
				offsets: (0 @ tabsPane height corner: 0 @ 0)).

	aBin color: (Color gray muchLighter);
		setNameTo: 'Objects' translated.

	self color: (Color gray);
		setNameTo: 'Objects' translated.

	self partsBin removeAllMorphs.
	self initializeWithTabs: self tabsForCategories.

	"Select the first category"
	(self tabsPane submorphs at: 1 ifAbsent: [nil])
		ifNotNil: [:sm | sm doButtonAction]! !

!PESupplies methodsFor: 'initialization' stamp: 'Richo 4/8/2011 14:05'!
initializeWithTabs: tabList
	"Initialize the receiver to have the given tabs"
	| oldPane newPane |
	oldPane := self tabsPane.
	newPane := (self paneForTabs: tabList)
		setNameTo: 'TabPane';
		yourself.
	newPane layoutFrame: oldPane layoutFrame.
	self replaceSubmorph: oldPane by: newPane.
	newPane layoutChanged; fullBounds.

! !

!PESupplies methodsFor: 'categories' stamp: 'Richo 4/8/2011 14:01'!
installQuads: quads fromButton: aButton
	"Install items in the bottom pane that correspond to the given set of quads, as triggered from the given button"

	| aPartsBin sortedQuads oldResizing |
	aPartsBin _ self partsBin.
	oldResizing := aPartsBin vResizing.
	aPartsBin removeAllMorphs.
	sortedQuads _ (PartsBin translatedQuads: quads)
							asSortedCollection: [:a :b | a third < b third].
	aPartsBin listDirection: #leftToRight quadList: sortedQuads.
	aButton ifNotNil: [self tabsPane highlightOnlySubmorph: aButton].
	aPartsBin vResizing: oldResizing.
	aPartsBin layoutChanged; fullBounds.! !

!PESupplies methodsFor: 'update' stamp: 'Richo 4/3/2012 12:47'!
localeChanged
	"Restore Physical Etoys supplies flap"
	"It has to be wrapped in a deferred UI message, otherwise it
	doesn't work. I'm not really sure why, but I would think it adds
	the PE flap in the middle of the process and then it deletes it"
	WorldState addDeferredUIMessage: [PEUtilities makePESuppliesFlap]! !

!PESupplies methodsFor: 'tabs' stamp: 'Richo 4/8/2011 14:05'!
paneForTabs: tabList 
	"Answer a pane bearing tabs for the given list"
	tabList do: [:t |
			t color: Color transparent.
			t borderWidth: 1;
				borderColor: Color black].

	^ AlignmentMorph newRow
			listDirection: #leftToRight;
			wrapDirection: #topToBottom;
			vResizing: #spaceFill;
			hResizing: #spaceFill;
			cellInset: 6;
			layoutInset: 4;
			listCentering: #center;
			listSpacing: #equal;
			addAllMorphs: tabList;
			color: Color gray lighter lighter;
			width: self layoutBounds width;
			yourself! !

!PESupplies methodsFor: 'submorph access' stamp: 'Richo 4/8/2011 13:51'!
partsBin
	^self findDeeplyA: PartsBin.! !

!PESupplies methodsFor: 'categories' stamp: 'Richo 4/8/2011 14:11'!
showCategory: aCategoryName fromButton: aButton 
	"Project items from the given category into my lower pane"
	| quads |
	self partsBin removeAllMorphs.
	Cursor wait
		showWhile: [quads := OrderedCollection new.
			Morph withAllSubclasses
				do: [:aClass | aClass theNonMetaClass
						addPartsDescriptorQuadsTo: quads
						if: [:aDescription | aDescription categories includes: aCategoryName]].
			quads := quads
						asSortedCollection: [:q1 :q2 | q1 third <= q2 third].
			self installQuads: quads fromButton: aButton	]! !

!PESupplies methodsFor: 'categories' stamp: 'Richo 4/12/2011 12:33'!
tabsForCategories
	"Answer a list of buttons which, when hit, will
	trigger the choice of a category"
	| buttonList aButton categoryList add |
	categoryList := OrderedCollection new.
	add := [:cat | (categoryList includes: cat)
				ifFalse: [categoryList add: cat]].
	PEUtilities packages
		reject: [:each | #('PhysicalEtoys' ) includes: each]
		thenDo: [:packageName | (PackageInfo named: packageName) classes
				select: [:each | each inheritsFrom: Morph]
				thenDo: [:aClass | (aClass class includesSelector: #descriptionForPartsBin)
						ifTrue: [(aClass class includesSelector: #descriptionForPartsBin)
								ifTrue: [aClass descriptionForPartsBin categories do: add].
							(aClass class includesSelector: #supplementaryPartsDescriptions)
								ifTrue: [aClass supplementaryPartsDescriptions
										do: [:aDescription | aDescription categories do: add]]]]].
	buttonList := categoryList
				collect: [:catName | 
					aButton := SimpleButtonMorph new label: catName translated font: Preferences standardEToysButtonFont.
					aButton actWhen: #buttonDown.
					aButton target: self;
						 actionSelector: #showCategory:fromButton:;
						 arguments: {catName. aButton}].
	^ buttonList! !

!PESupplies methodsFor: 'submorph access' stamp: 'Richo 4/8/2011 13:52'!
tabsPane
	"Answer the receiver's tabs pane, nil if none"

	^ self submorphNamed: 'TabPane' ifNone: [].! !

SymbolListType subclass: #PluggableListType
	instanceVariableNames: 'block'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PhysicalEtoys-Utilities'!

!PluggableListType methodsFor: 'accessing' stamp: 'Richo 5/3/2011 13:11'!
block: aBlock
	block := aBlock! !

!PluggableListType methodsFor: 'accessing' stamp: 'Richo 5/3/2011 13:12'!
choices
	"answer the list of choices to offer as variant values"

	^ block value! !

!ScriptEditorMorph methodsFor: '*physicaletoys-compilersupport' stamp: 'Richo 3/19/2012 13:34'!
buttonRowForEditor
	"Answer a row of buttons that comprise the header at the top of the Scriptor"

	| aRow aString aStatusMorph aButton aTile aMorph goldBoxButton aBox |
	aRow _ AlignmentMorph newRow color: ScriptingSystem baseColor; layoutInset: 1.
	aRow hResizing: #spaceFill.
	aRow vResizing: #shrinkWrap.
	self addDismissButtonTo: aRow.
	aRow addTransparentSpacerOfSize: 9.

	"Player's name"
	aString _ playerScripted externalName.
	aMorph _ StringMorph contents: aString font: ScriptingSystem fontForTiles.
	aMorph setNameTo: 'title'.
	aRow addMorphBack: aMorph.
	aRow addTransparentSpacerOfSize: 6.

	"Script's name"
	aBox := AlignmentMorph newRow.
	aBox hResizing: #shrinkWrap; vResizing: #shrinkWrap.
	aBox color: (Color r: 0.839 g: 1.0 b: 0.806).
	aBox borderWidth: 1.
	aBox  borderColor: (Color r: 0.645 g: 0.774 b: 0.613).
	aButton _ UpdatingStringMorph new.
	aButton useStringFormat;
		target:  self;
		getSelector: #scriptTitle;
		setNameTo: 'script name';
		font: ScriptingSystem fontForNameEditingInScriptor;
		putSelector: #setScriptNameTo:;
		setProperty: #okToTextEdit toValue: true;
		step;
		yourself.
	aBox addMorph: aButton.
	aRow addMorphBack: aBox.
	aBox setBalloonText: 'Click here to edit the name of the script.' translated.
	"aRow addTransparentSpacerOfSize: 9."
	aRow addVariableTransparentSpacer.

	"Try It button"
	(self playerScripted isUsingCompiledMode or: [self hasParameter]) ifFalse:
		[aRow addMorphBack:
			((ThreePhaseButtonMorph
				labelSymbol: #TryIt
				target: self
				actionSelector: #tryMe
				arguments: EmptyArray)
				actWhen: #whilePressed;
				balloonTextSelector: #tryMe).
		aRow addTransparentSpacerOfSize: 3].

	"Step button"
	(self playerScripted isUsingCompiledMode or: [self hasParameter]) ifFalse:
		[aRow addMorphBack: (aButton := ThreePhaseButtonMorph
				labelSymbol: #StepMe
				target: self
				actionSelector: #stepMe
				arguments: EmptyArray).
		aButton balloonTextSelector: #stepMe.
		aRow addTransparentSpacerOfSize: 3].

	"Status controller"
	self hasParameter
		ifTrue:
			[aTile _ TypeListTile new choices: Vocabulary typeChoicesForUserVariables dataType: nil.
			aTile addArrows.
			aTile setLiteral: self typeForParameter.
			aRow addMorphBack: aTile.
			aTile borderColor: Color red.
			aTile color: ScriptingSystem uniformTileInteriorColor.
			aTile setBalloonText: 'Drag from here to get a parameter tile' translated.
			aTile addCaretsAsAppropriate: true]
		ifFalse:
			[aRow addMorphBack: (aStatusMorph _ self scriptInstantiation statusControlMorph)].

	"aRow addTransparentSpacerOfSize: 3."
	aRow addVariableTransparentSpacer.

	"Gold-box"
	aRow addMorphBack: (goldBoxButton _ IconicButton new).
	goldBoxButton borderWidth: 0;
			labelGraphic: (ScriptingSystem formAtKey: 'RoundGoldBox'); color: Color transparent; 
			actWhen: #buttonDown;
			target: self;
			actionSelector: #offerGoldBoxMenu;
			shedSelvedge;
			setBalloonText: 'click here to get a palette of useful tiles to use in your script.' translated.
	aRow addTransparentSpacerOfSize: 6@1.

	"Menu Button"
	aButton _ self menuButton.
	aButton actionSelector: #offerScriptorMenu.
	aRow addMorphBack: aButton.

	(playerScripted existingScriptInstantiationForSelector: scriptName)
		ifNotNilDo:
			[:inst | inst updateStatusMorph: aStatusMorph].
	^ aRow! !

!FileDirectory methodsFor: '*physicaletoys' stamp: 'Richo 3/8/2012 16:57'!
createTemporaryDirectory
	| randomName |
	[randomName := (String new: 20) 
		collect: [:ign | Character value: ((65 to: 90), (97 to: 122)) atRandom]]
			doWhileTrue: [self directoryExists: randomName].
	^ self createDirectory: randomName;
		directoryNamed: randomName! !

!FileDirectory methodsFor: '*physicaletoys' stamp: 'Richo 10/26/2011 12:13'!
fileNamed: localFileName do: aBlock
	"Open the file with the given name in this directory for writing and pass it as argument to aBlock."

	^ FileStream concreteStream fileNamed: (self fullNameFor: localFileName) do: aBlock
! !

!FileDirectory methodsFor: '*physicaletoys' stamp: 'Richo 10/21/2011 18:44'!
forceNewFileNamed: localFileName do: aBlock
	"Open the file with the given name in this directory for writing.  If it already exists, delete it first without asking and pass it as argument to aBlock."

	^ FileStream concreteStream forceNewFileNamed: (self fullNameFor: localFileName) do: aBlock
! !

!FileDirectory methodsFor: '*physicaletoys' stamp: 'Richo 3/8/2012 16:51'!
temporaryDirectoryDo: aBlock
	| tempDir |
	tempDir := self createTemporaryDirectory.
	[aBlock value: tempDir]
		ensure: [tempDir recursiveDelete]! !

!StandardViewer methodsFor: '*PhysicalEtoys-categories' stamp: 'Richo 12/23/2009 16:15'!
addCategoryViewer
| likely |
likely := self likelyCategoryToShow.
(self categoriesCurrentlyShowing includes: likely)
	ifFalse: [self addCategoryViewerFor: likely]
! !

!TextMorph methodsFor: '*PhysicalEtoys-Spreadsheet' stamp: 'Richo 4/12/2011 16:56'!
setExtentFromHalo: aPoint
	super setExtentFromHalo: aPoint.
	self triggerEvent: #extentChange with: self! !

!ByteArray methodsFor: '*PhysicalEtoys-FFI fix' stamp: 'hg 2/28/2000 13:56'!
structAt: byteOffset length: length
	"Return a structure of the given length starting at the indicated byte offset."
	| value |
	value := ByteArray new: length.
	1 to: length do:[:i|
		value unsignedByteAt: i put: (self unsignedByteAt: byteOffset+i-1)].
	^value! !

!CategoryViewer methodsFor: '*physicaletoys-compilersupport' stamp: 'Richo 1/27/2011 14:58'!
phraseForCommandFrom: aMethodInterface 
	"Answer a phrase for the non-slot-like command
	represented by aMethodInterface - classic tiles"
	| aRow resultType cmd names argType argTile selfTile aPhrase balloonTextSelector stat inst aDocString universal tileBearingHelp |
	aDocString := aMethodInterface documentation.
	aDocString = 'no help available'
		ifTrue: [aDocString := nil].
	names := scriptedPlayer class namedTileScriptSelectors.
	resultType := aMethodInterface resultType.
	cmd := aMethodInterface selector.
	(universal := scriptedPlayer isUniversalTiles)
		ifTrue: [aPhrase := scriptedPlayer universalTilesForInterface: aMethodInterface]
		ifFalse: [cmd numArgs == 0
				ifTrue: [aPhrase := PhraseTileMorph new vocabulary: self currentVocabulary.
					aPhrase
						setOperator: cmd
						type: resultType
						rcvrType: #Player]
				ifFalse: ["only one arg supported in classic
					tiles, so if this is fed
					with a selector with > 1 arg, results
					will be very strange"
					argType := aMethodInterface typeForArgumentNumber: 1.
					aPhrase := PhraseTileMorph new vocabulary: self currentVocabulary.
					(self isSpecialPatchReceiver: scriptedPlayer and: cmd)
						ifTrue: [aPhrase
								setOperator: cmd
								type: resultType
								rcvrType: #Patch
								argType: argType]
						ifFalse: [aPhrase
								setOperator: cmd
								type: resultType
								rcvrType: #Player
								argType: argType].
					(self isSpecialPatchCase: scriptedPlayer and: cmd)
						ifTrue: [argTile := (Vocabulary vocabularyForType: argType)
										defaultArgumentTileFor: scriptedPlayer]
						ifFalse: [argTile := ScriptingSystem tileForArgType: argType forCommand: cmd].
					(#(#bounce: #wrap: ) includes: cmd)
						ifTrue: ["help for the embattled bj"
							argTile setLiteral: 'silence';
								 updateLiteralLabel].
					argTile position: aPhrase lastSubmorph position.
					aPhrase lastSubmorph addMorph: argTile]].
	(scriptedPlayer slotInfo includesKey: cmd)
		ifTrue: [balloonTextSelector := #userSlot].
	(scriptedPlayer belongsToUniClass
			and: [scriptedPlayer class includesSelector: cmd])
		ifTrue: [aDocString
				ifNil: [aDocString := (scriptedPlayer class userScriptForPlayer: scriptedPlayer selector: cmd) documentation].
			aDocString
				ifNil: [balloonTextSelector := #userScript]].
	tileBearingHelp := universal
				ifTrue: [aPhrase submorphs second]
				ifFalse: [aPhrase operatorTile].
	aDocString
		ifNil: [balloonTextSelector
				ifNil: [tileBearingHelp setProperty: #inherentSelector toValue: cmd.
					balloonTextSelector := nil].
			tileBearingHelp balloonTextSelector: balloonTextSelector]
		ifNotNil: [tileBearingHelp setBalloonText: aDocString].
	aPhrase markAsPartsDonor.
	cmd == #emptyScript
		ifTrue: [aPhrase setProperty: #newPermanentScript toValue: true.
			aPhrase setProperty: #newPermanentPlayer toValue: scriptedPlayer.
			aPhrase submorphs second setBalloonText: 'drag and drop to 
add a new script' translated].
	universal
		ifFalse: [selfTile := self tileForSelf.
			selfTile position: aPhrase firstSubmorph position.
			aPhrase firstSubmorph addMorph: selfTile].
	aRow := ViewerLine newRow.
	aRow elementSymbol: cmd asSymbol.
	aRow
		addMorphBack: (balloonTextSelector = #userSlot
				ifTrue: [self infoButtonFor: cmd]
				ifFalse: [(cmd = #emptyScript
							or: [(names includes: cmd)
									and: [scriptedPlayer isUsingCompiledMode]])
						ifTrue: [ScriptingSystem buttonSpacer]
						ifFalse: [ScriptingSystem tryButtonFor: aPhrase]]).
	aRow addMorphBack: self spacerAfterButton.
	aRow addMorphBack: aPhrase.
	"aPhrase on: #mouseEnter send:
	#addCommandFeedback to: aRow.
	aPhrase on: #mouseLeave send:
	#removeHighlightFeedback to: aRow.
	aPhrase on: #mouseLeaveDragging send:
	#removeHighlightFeedback to: aRow."
	(names includes: cmd)
		ifTrue: [aPhrase userScriptSelector: cmd.
			cmd numArgs == 0
				ifTrue: [aPhrase beTransparent.
					aRow addMorphBack: AlignmentMorph newVariableTransparentSpacer.
					aRow addMorphBack: (stat := (inst := scriptedPlayer scriptInstantiationForSelector: cmd) statusControlMorph).
					inst updateStatusMorph: stat]].
	aRow beSticky; disableDragNDrop.
	^ aRow! !

!ProjectNavigationMorph methodsFor: '*physicaletoys' stamp: 'Richo 4/10/2012 16:47'!
findAProjectSimple
	((PEUtilities isPEProject: Project current)
		ifTrue: [PEFileList]
		ifFalse: [FileList2])
			morphicViewProjectLoader2InWorld: self world 
			reallyLoad: true
			dirFilterType: #limitedSuperSwikiDirectoryList! !

!ProjectNavigationMorph methodsFor: '*physicaletoys' stamp: 'Richo 4/10/2012 16:48'!
publishProject

	self world paintBoxOrNil ifNotNil: [
		(self confirm: 'You seem to be painting a sketch.
Do you continue and publish the project with the paint tool?' translated) ifFalse: [^ self].
	].
	"Only show the old dialog if the project is not physical etoys
	and it's clean (safe for the squeakland showcase)"
	((PEUtilities isPEProject: Project current) not
	and: [PEUtilities isEtoysProjectClean: Project current])
		ifTrue: ["Old behavior"
			self 
				publishStyle: #limitedSuperSwikiPublishDirectoryList 
				forgetURL: false
				withRename: false]
		ifFalse: [Project current storeOnServerAsPE]! !
PEInstaller initialize!
PEModule initialize!
PEProject initialize!
PESerialPort initialize!
PETranslator initialize!
PEUtilities initialize!
BasicSpreadsheetMorph initialize!
